# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import Remlfit


def test_Remlfit_inputs():
    input_map = dict(
        STATmask=dict(
            argstr="-STATmask %s",
            extensions=None,
        ),
        addbase=dict(
            argstr="-addbase %s",
            copyfile=False,
            sep=" ",
        ),
        args=dict(
            argstr="%s",
        ),
        automask=dict(
            argstr="-automask",
            usedefault=True,
        ),
        dsort=dict(
            argstr="-dsort %s",
            copyfile=False,
            extensions=None,
        ),
        dsort_nods=dict(
            argstr="-dsort_nods",
            requires=["dsort"],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        errts_file=dict(
            argstr="-Rerrts %s",
            extensions=None,
        ),
        fitts_file=dict(
            argstr="-Rfitts %s",
            extensions=None,
        ),
        fout=dict(
            argstr="-fout",
        ),
        glt_file=dict(
            argstr="-Rglt %s",
            extensions=None,
        ),
        gltsym=dict(
            argstr='-gltsym "%s" %s...',
        ),
        goforit=dict(
            argstr="-GOFORIT",
        ),
        in_files=dict(
            argstr='-input "%s"',
            copyfile=False,
            mandatory=True,
            sep=" ",
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        matim=dict(
            argstr="-matim %s",
            extensions=None,
            xor=["matrix"],
        ),
        matrix=dict(
            argstr="-matrix %s",
            extensions=None,
            mandatory=True,
        ),
        nobout=dict(
            argstr="-nobout",
        ),
        nodmbase=dict(
            argstr="-nodmbase",
            requires=["addbase", "dsort"],
        ),
        nofdr=dict(
            argstr="-noFDR",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        obeta=dict(
            argstr="-Obeta %s",
            extensions=None,
        ),
        obuck=dict(
            argstr="-Obuck %s",
            extensions=None,
        ),
        oerrts=dict(
            argstr="-Oerrts %s",
            extensions=None,
        ),
        ofitts=dict(
            argstr="-Ofitts %s",
            extensions=None,
        ),
        oglt=dict(
            argstr="-Oglt %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="-Rbuck %s",
            extensions=None,
        ),
        outputtype=dict(),
        ovar=dict(
            argstr="-Ovar %s",
            extensions=None,
        ),
        polort=dict(
            argstr="-polort %d",
            xor=["matrix"],
        ),
        quiet=dict(
            argstr="-quiet",
        ),
        rbeta_file=dict(
            argstr="-Rbeta %s",
            extensions=None,
        ),
        rout=dict(
            argstr="-rout",
        ),
        slibase=dict(
            argstr="-slibase %s",
        ),
        slibase_sm=dict(
            argstr="-slibase_sm %s",
        ),
        tout=dict(
            argstr="-tout",
        ),
        usetemp=dict(
            argstr="-usetemp",
        ),
        var_file=dict(
            argstr="-Rvar %s",
            extensions=None,
        ),
        verb=dict(
            argstr="-verb",
        ),
        wherr_file=dict(
            argstr="-Rwherr %s",
            extensions=None,
        ),
    )
    inputs = Remlfit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Remlfit_outputs():
    output_map = dict(
        errts_file=dict(
            extensions=None,
        ),
        fitts_file=dict(
            extensions=None,
        ),
        glt_file=dict(
            extensions=None,
        ),
        obeta=dict(
            extensions=None,
        ),
        obuck=dict(
            extensions=None,
        ),
        oerrts=dict(
            extensions=None,
        ),
        ofitts=dict(
            extensions=None,
        ),
        oglt=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
        ovar=dict(
            extensions=None,
        ),
        rbeta_file=dict(
            extensions=None,
        ),
        var_file=dict(
            extensions=None,
        ),
        wherr_file=dict(
            extensions=None,
        ),
    )
    outputs = Remlfit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Slicer


def test_Slicer_inputs():
    input_map = dict(
        all_axial=dict(
            argstr="-A",
            position=10,
            requires=["image_width"],
            xor=("single_slice", "middle_slices", "all_axial", "sample_axial"),
        ),
        args=dict(
            argstr="%s",
        ),
        colour_map=dict(
            argstr="-l %s",
            extensions=None,
            position=4,
        ),
        dither_edges=dict(
            argstr="-t",
            position=7,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        image_edges=dict(
            argstr="%s",
            extensions=None,
            position=2,
        ),
        image_width=dict(
            argstr="%d",
            position=-2,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        intensity_range=dict(
            argstr="-i %.3f %.3f",
            position=5,
        ),
        label_slices=dict(
            argstr="-L",
            position=3,
            usedefault=True,
        ),
        middle_slices=dict(
            argstr="-a",
            position=10,
            xor=("single_slice", "middle_slices", "all_axial", "sample_axial"),
        ),
        nearest_neighbour=dict(
            argstr="-n",
            position=8,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            position=-1,
        ),
        output_type=dict(),
        sample_axial=dict(
            argstr="-S %d",
            position=10,
            requires=["image_width"],
            xor=("single_slice", "middle_slices", "all_axial", "sample_axial"),
        ),
        scaling=dict(
            argstr="-s %f",
            position=0,
        ),
        show_orientation=dict(
            argstr="%s",
            position=9,
            usedefault=True,
        ),
        single_slice=dict(
            argstr="-%s",
            position=10,
            requires=["slice_number"],
            xor=("single_slice", "middle_slices", "all_axial", "sample_axial"),
        ),
        slice_number=dict(
            argstr="-%d",
            position=11,
        ),
        threshold_edges=dict(
            argstr="-e %.3f",
            position=6,
        ),
    )
    inputs = Slicer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Slicer_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Slicer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import NwarpApply


def test_NwarpApply_inputs():
    input_map = dict(
        ainterp=dict(
            argstr="-ainterp %s",
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-source %s",
            mandatory=True,
        ),
        interp=dict(
            argstr="-interp %s",
            usedefault=True,
        ),
        inv_warp=dict(
            argstr="-iwarp",
        ),
        master=dict(
            argstr="-master %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_Nwarp",
        ),
        quiet=dict(
            argstr="-quiet",
            xor=["verb"],
        ),
        short=dict(
            argstr="-short",
        ),
        verb=dict(
            argstr="-verb",
            xor=["quiet"],
        ),
        warp=dict(
            argstr="-nwarp %s",
            mandatory=True,
        ),
    )
    inputs = NwarpApply.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_NwarpApply_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = NwarpApply.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Miscellaneous file manipulation functions
"""
import sys
import pickle
import errno
import subprocess as sp
import gzip
import hashlib
import locale
from hashlib import md5
import os
import os.path as op
import re
import shutil
import contextlib
import posixpath
from pathlib import Path
import simplejson as json
from time import sleep, time

from .. import logging, config, __version__ as version
from .misc import is_container

fmlogger = logging.getLogger("nipype.utils")

related_filetype_sets = [(".hdr", ".img", ".mat"), (".nii", ".mat"), (".BRIK", ".HEAD")]


def _resolve_with_filenotfound(path, **kwargs):
    """ Raise FileNotFoundError instead of OSError """
    try:
        return path.resolve(**kwargs)
    except OSError as e:
        if isinstance(e, FileNotFoundError):
            raise
        raise FileNotFoundError(str(path))


def path_resolve(path, strict=False):
    try:
        return _resolve_with_filenotfound(path, strict=strict)
    except TypeError:  # PY35
        pass

    path = path.absolute()
    if strict or path.exists():
        return _resolve_with_filenotfound(path)

    # This is a hacky shortcut, using path.absolute() unmodified
    # In cases where the existing part of the path contains a
    # symlink, different results will be produced
    return path


def split_filename(fname):
    """Split a filename into parts: path, base filename and extension.

    Parameters
    ----------
    fname : str
        file or path name

    Returns
    -------
    pth : str
        base path from fname
    fname : str
        filename from fname, without extension
    ext : str
        file extension from fname

    Examples
    --------
    >>> from nipype.utils.filemanip import split_filename
    >>> pth, fname, ext = split_filename('/home/data/subject.nii.gz')
    >>> pth
    '/home/data'

    >>> fname
    'subject'

    >>> ext
    '.nii.gz'

    """

    special_extensions = [".nii.gz", ".tar.gz", ".niml.dset"]

    pth = op.dirname(fname)
    fname = op.basename(fname)

    ext = None
    for special_ext in special_extensions:
        ext_len = len(special_ext)
        if (len(fname) > ext_len) and (fname[-ext_len:].lower() == special_ext.lower()):
            ext = fname[-ext_len:]
            fname = fname[:-ext_len]
            break
    if not ext:
        fname, ext = op.splitext(fname)

    return pth, fname, ext


def fname_presuffix(fname, prefix="", suffix="", newpath=None, use_ext=True):
    """Manipulates path and name of input filename

    Parameters
    ----------
    fname : string
        A filename (may or may not include path)
    prefix : string
        Characters to prepend to the filename
    suffix : string
        Characters to append to the filename
    newpath : string
        Path to replace the path of the input fname
    use_ext : boolean
        If True (default), appends the extension of the original file
        to the output name.

    Returns
    -------
    Absolute path of the modified filename

    >>> from nipype.utils.filemanip import fname_presuffix
    >>> fname = 'foo.nii.gz'
    >>> fname_presuffix(fname,'pre','post','/tmp')
    '/tmp/prefoopost.nii.gz'

    >>> from nipype.interfaces.base import Undefined
    >>> fname_presuffix(fname, 'pre', 'post', Undefined) == \
            fname_presuffix(fname, 'pre', 'post')
    True

    """
    pth, fname, ext = split_filename(fname)
    if not use_ext:
        ext = ""

    # No need for isdefined: bool(Undefined) evaluates to False
    if newpath:
        pth = op.abspath(newpath)
    return op.join(pth, prefix + fname + suffix + ext)


def fnames_presuffix(fnames, prefix="", suffix="", newpath=None, use_ext=True):
    """Calls fname_presuffix for a list of files.
    """
    f2 = []
    for fname in fnames:
        f2.append(fname_presuffix(fname, prefix, suffix, newpath, use_ext))
    return f2


def hash_rename(filename, hashvalue):
    """renames a file given original filename and hash
    and sets path to output_directory
    """
    path, name, ext = split_filename(filename)
    newfilename = "".join((name, "_0x", hashvalue, ext))
    return op.join(path, newfilename)


def check_forhash(filename):
    """checks if file has a hash in its filename"""
    if isinstance(filename, list):
        filename = filename[0]
    path, name = op.split(filename)
    if re.search("(_0x[a-z0-9]{32})", name):
        hashvalue = re.findall("(_0x[a-z0-9]{32})", name)
        return True, hashvalue
    else:
        return False, None


def hash_infile(afile, chunk_len=8192, crypto=hashlib.md5, raise_notfound=False):
    """
    Computes hash of a file using 'crypto' module

    >>> hash_infile('smri_ants_registration_settings.json')
    'f225785dfb0db9032aa5a0e4f2c730ad'

    >>> hash_infile('surf01.vtk')
    'fdf1cf359b4e346034372cdeb58f9a88'

    >>> hash_infile('spminfo')
    '0dc55e3888c98a182dab179b976dfffc'

    >>> hash_infile('fsl_motion_outliers_fd.txt')
    'defd1812c22405b1ee4431aac5bbdd73'


    """
    if not op.isfile(afile):
        if raise_notfound:
            raise RuntimeError('File "%s" not found.' % afile)
        return None

    crypto_obj = crypto()
    with open(afile, "rb") as fp:
        while True:
            data = fp.read(chunk_len)
            if not data:
                break
            crypto_obj.update(data)
    return crypto_obj.hexdigest()


def hash_timestamp(afile):
    """ Computes md5 hash of the timestamp of a file """
    md5hex = None
    if op.isfile(afile):
        md5obj = md5()
        stat = os.stat(afile)
        md5obj.update(str(stat.st_size).encode())
        md5obj.update(str(stat.st_mtime).encode())
        md5hex = md5obj.hexdigest()
    return md5hex


def _parse_mount_table(exit_code, output):
    """Parses the output of ``mount`` to produce (path, fs_type) pairs

    Separated from _generate_cifs_table to enable testing logic with real
    outputs
    """
    # Not POSIX
    if exit_code != 0:
        return []

    # Linux mount example:  sysfs on /sys type sysfs (rw,nosuid,nodev,noexec)
    #                          <PATH>^^^^      ^^^^^<FSTYPE>
    # OSX mount example:    /dev/disk2 on / (hfs, local, journaled)
    #                               <PATH>^  ^^^<FSTYPE>
    pattern = re.compile(r".*? on (/.*?) (?:type |\()([^\s,\)]+)")

    # Keep line and match for error reporting (match == None on failure)
    # Ignore empty lines
    matches = [(l, pattern.match(l)) for l in output.strip().splitlines() if l]

    # (path, fstype) tuples, sorted by path length (longest first)
    mount_info = sorted(
        (match.groups() for _, match in matches if match is not None),
        key=lambda x: len(x[0]),
        reverse=True,
    )
    cifs_paths = [path for path, fstype in mount_info if fstype.lower() == "cifs"]

    # Report failures as warnings
    for line, match in matches:
        if match is None:
            fmlogger.debug("Cannot parse mount line: '%s'", line)

    return [
        mount
        for mount in mount_info
        if any(mount[0].startswith(path) for path in cifs_paths)
    ]


def _generate_cifs_table():
    """Construct a reverse-length-ordered list of mount points that
    fall under a CIFS mount.

    This precomputation allows efficient checking for whether a given path
    would be on a CIFS filesystem.

    On systems without a ``mount`` command, or with no CIFS mounts, returns an
    empty list.
    """
    exit_code, output = sp.getstatusoutput("mount")
    return _parse_mount_table(exit_code, output)


_cifs_table = _generate_cifs_table()


def on_cifs(fname):
    """
    Checks whether a file path is on a CIFS filesystem mounted in a POSIX
    host (i.e., has the ``mount`` command).

    On Windows, Docker mounts host directories into containers through CIFS
    shares, which has support for Minshall+French symlinks, or text files that
    the CIFS driver exposes to the OS as symlinks.
    We have found that under concurrent access to the filesystem, this feature
    can result in failures to create or read recently-created symlinks,
    leading to inconsistent behavior and ``FileNotFoundError``.

    This check is written to support disabling symlinks on CIFS shares.

    """
    # Only the first match (most recent parent) counts
    for fspath, fstype in _cifs_table:
        if fname.startswith(fspath):
            return fstype == "cifs"
    return False


def copyfile(
    originalfile,
    newfile,
    copy=False,
    create_new=False,
    hashmethod=None,
    use_hardlink=False,
    copy_related_files=True,
):
    """Copy or link ``originalfile`` to ``newfile``.

    If ``use_hardlink`` is True, and the file can be hard-linked, then a
    link is created, instead of copying the file.

    If a hard link is not created and ``copy`` is False, then a symbolic
    link is created.

    Parameters
    ----------
    originalfile : str
        full path to original file
    newfile : str
        full path to new file
    copy : Bool
        specifies whether to copy or symlink files
        (default=False) but only for POSIX systems
    use_hardlink : Bool
        specifies whether to hard-link files, when able
        (Default=False), taking precedence over copy
    copy_related_files : Bool
        specifies whether to also operate on related files, as defined in
        ``related_filetype_sets``

    Returns
    -------
    None

    """
    newhash = None
    orighash = None
    fmlogger.debug(newfile)

    if create_new:
        while op.exists(newfile):
            base, fname, ext = split_filename(newfile)
            s = re.search("_c[0-9]{4,4}$", fname)
            i = 0
            if s:
                i = int(s.group()[2:]) + 1
                fname = fname[:-6] + "_c%04d" % i
            else:
                fname += "_c%04d" % i
            newfile = base + os.sep + fname + ext

    if hashmethod is None:
        hashmethod = config.get("execution", "hash_method").lower()

    # Don't try creating symlinks on CIFS
    if copy is False and on_cifs(newfile):
        copy = True

    # Existing file
    # -------------
    # Options:
    #   symlink
    #       to regular file originalfile            (keep if symlinking)
    #       to same dest as symlink originalfile    (keep if symlinking)
    #       to other file                           (unlink)
    #   regular file
    #       hard link to originalfile               (keep)
    #       copy of file (same hash)                (keep)
    #       different file (diff hash)              (unlink)
    keep = False
    if op.lexists(newfile):
        if op.islink(newfile):
            if all(
                (
                    os.readlink(newfile) == op.realpath(originalfile),
                    not use_hardlink,
                    not copy,
                )
            ):
                keep = True
        elif posixpath.samefile(newfile, originalfile):
            keep = True
        else:
            if hashmethod == "timestamp":
                hashfn = hash_timestamp
            elif hashmethod == "content":
                hashfn = hash_infile
            else:
                raise AttributeError("Unknown hash method found:", hashmethod)
            newhash = hashfn(newfile)
            fmlogger.debug(
                "File: %s already exists,%s, copy:%d", newfile, newhash, copy
            )
            orighash = hashfn(originalfile)
            keep = newhash == orighash
        if keep:
            fmlogger.debug(
                "File: %s already exists, not overwriting, copy:%d", newfile, copy
            )
        else:
            os.unlink(newfile)

    # New file
    # --------
    # use_hardlink & can_hardlink => hardlink
    # ~hardlink & ~copy & can_symlink => symlink
    # ~hardlink & ~symlink => copy
    if not keep and use_hardlink:
        try:
            fmlogger.debug("Linking File: %s->%s", newfile, originalfile)
            # Use realpath to avoid hardlinking symlinks
            os.link(op.realpath(originalfile), newfile)
        except OSError:
            use_hardlink = False  # Disable hardlink for associated files
        else:
            keep = True

    if not keep and not copy and os.name == "posix":
        try:
            fmlogger.debug("Symlinking File: %s->%s", newfile, originalfile)
            os.symlink(originalfile, newfile)
        except OSError:
            copy = True  # Disable symlink for associated files
        else:
            keep = True

    if not keep:
        try:
            fmlogger.debug("Copying File: %s->%s", newfile, originalfile)
            shutil.copyfile(originalfile, newfile)
        except shutil.Error as e:
            fmlogger.warning(str(e))

    # Associated files
    if copy_related_files:
        related_file_pairs = (
            get_related_files(f, include_this_file=False)
            for f in (originalfile, newfile)
        )
        for alt_ofile, alt_nfile in zip(*related_file_pairs):
            if op.exists(alt_ofile):
                copyfile(
                    alt_ofile,
                    alt_nfile,
                    copy,
                    hashmethod=hashmethod,
                    use_hardlink=use_hardlink,
                    copy_related_files=False,
                )

    return newfile


def get_related_files(filename, include_this_file=True):
    """Returns a list of related files, as defined in
    ``related_filetype_sets``, for a filename. (e.g., Nifti-Pair, Analyze (SPM)
    and AFNI files).

    Parameters
    ----------
    filename : str
        File name to find related filetypes of.
    include_this_file : bool
        If true, output includes the input filename.
    """
    related_files = []
    path, name, this_type = split_filename(filename)
    for type_set in related_filetype_sets:
        if this_type in type_set:
            for related_type in type_set:
                if include_this_file or related_type != this_type:
                    related_files.append(op.join(path, name + related_type))
    if not len(related_files):
        related_files = [filename]
    return related_files


def copyfiles(filelist, dest, copy=False, create_new=False):
    """Copy or symlink files in ``filelist`` to ``dest`` directory.

    Parameters
    ----------
    filelist : list
        List of files to copy.
    dest : path/files
        full path to destination. If it is a list of length greater
        than 1, then it assumes that these are the names of the new
        files.
    copy : Bool
        specifies whether to copy or symlink files
        (default=False) but only for posix systems

    Returns
    -------
    None

    """
    outfiles = ensure_list(dest)
    newfiles = []
    for i, f in enumerate(ensure_list(filelist)):
        if isinstance(f, list):
            newfiles.insert(i, copyfiles(f, dest, copy=copy, create_new=create_new))
        else:
            if len(outfiles) > 1:
                destfile = outfiles[i]
            else:
                destfile = fname_presuffix(f, newpath=outfiles[0])
            destfile = copyfile(f, destfile, copy, create_new=create_new)
            newfiles.insert(i, destfile)
    return newfiles


def ensure_list(filename):
    """Returns a list given either a string or a list
    """
    if isinstance(filename, (str, bytes)):
        return [filename]
    elif isinstance(filename, list):
        return filename
    elif is_container(filename):
        return [x for x in filename]
    else:
        return None


def simplify_list(filelist):
    """Returns a list if filelist is a list of length greater than 1,
       otherwise returns the first element
    """
    if len(filelist) > 1:
        return filelist
    else:
        return filelist[0]


filename_to_list = ensure_list
list_to_filename = simplify_list


def check_depends(targets, dependencies):
    """Return true if all targets exist and are newer than all dependencies.

    An OSError will be raised if there are missing dependencies.
    """
    tgts = ensure_list(targets)
    deps = ensure_list(dependencies)
    return all(map(op.exists, tgts)) and min(map(op.getmtime, tgts)) > max(
        list(map(op.getmtime, deps)) + [0]
    )


def save_json(filename, data):
    """Save data to a json file

    Parameters
    ----------
    filename : str
        Filename to save data in.
    data : dict
        Dictionary to save in json file.

    """
    mode = "w"
    with open(filename, mode) as fp:
        json.dump(data, fp, sort_keys=True, indent=4)


def load_json(filename):
    """Load data from a json file

    Parameters
    ----------
    filename : str
        Filename to load data from.

    Returns
    -------
    data : dict

    """

    with open(filename, "r") as fp:
        data = json.load(fp)
    return data


def loadcrash(infile, *args):
    if infile.endswith("pkl") or infile.endswith("pklz"):
        return loadpkl(infile)
    else:
        raise ValueError("Only pickled crashfiles are supported")


def loadpkl(infile):
    """Load a zipped or plain cPickled file."""
    infile = Path(infile)
    fmlogger.debug("Loading pkl: %s", infile)
    pklopen = gzip.open if infile.suffix == ".pklz" else open

    t = time()
    timeout = float(config.get("execution", "job_finished_timeout"))
    timed_out = True
    while (time() - t) < timeout:
        if infile.exists():
            timed_out = False
            break
        fmlogger.debug("'{}' missing; waiting 2s".format(infile))
        sleep(2)
    if timed_out:
        error_message = (
            "Result file {0} expected, but "
            "does not exist after ({1}) "
            "seconds.".format(infile, timeout)
        )
        raise IOError(error_message)

    with pklopen(str(infile), "rb") as pkl_file:
        pkl_contents = pkl_file.read()

    pkl_metadata = None

    # Look if pkl file contains version metadata
    idx = pkl_contents.find(b"\n")
    if idx >= 0:
        try:
            pkl_metadata = json.loads(pkl_contents[:idx])
        except (UnicodeDecodeError, json.JSONDecodeError):
            # Could not get version info
            pass
        else:
            # On success, skip JSON metadata
            pkl_contents = pkl_contents[idx + 1 :]

    # Pickle files may contain relative paths that must be resolved relative
    # to the working directory, so use indirectory while attempting to load
    unpkl = None
    try:
        with indirectory(infile.parent):
            unpkl = pickle.loads(pkl_contents)
    except UnicodeDecodeError:
        # Was this pickle created with Python 2.x?
        with indirectory(infile.parent):
            unpkl = pickle.loads(pkl_contents, fix_imports=True, encoding="utf-8")
        fmlogger.info("Successfully loaded pkl in compatibility mode.")
    # Unpickling problems
    except Exception as e:
        if pkl_metadata and "version" in pkl_metadata:
            if pkl_metadata["version"] != version:
                fmlogger.error(
                    """\
Attempted to open a results file generated by Nipype version %s, \
with an incompatible Nipype version (%s)""",
                    pkl_metadata["version"],
                    version,
                )
                raise e
        fmlogger.warning(
            """\
No metadata was found in the pkl file. Make sure you are currently using \
the same Nipype version from the generated pkl."""
        )
        raise e

    if unpkl is None:
        raise ValueError("Loading %s resulted in None." % infile)

    return unpkl


def crash2txt(filename, record):
    """ Write out plain text crash file """
    with open(filename, "w") as fp:
        if "node" in record:
            node = record["node"]
            fp.write("Node: {}\n".format(node.fullname))
            fp.write("Working directory: {}\n".format(node.output_dir()))
            fp.write("\n")
            fp.write("Node inputs:\n{}\n".format(node.inputs))
        fp.write("".join(record["traceback"]))


def read_stream(stream, logger=None, encoding=None):
    """
    Robustly reads a stream, sending a warning to a logger
    if some decoding error was raised.

    >>> read_stream(bytearray([65, 0xc7, 65, 10, 66]))  # doctest: +ELLIPSIS
    ['A...A', 'B']


    """
    default_encoding = encoding or locale.getdefaultlocale()[1] or "UTF-8"
    logger = logger or fmlogger
    try:
        out = stream.decode(default_encoding)
    except UnicodeDecodeError as err:
        out = stream.decode(default_encoding, errors="replace")
        logger.warning("Error decoding string: %s", err)
    return out.splitlines()


def savepkl(filename, record, versioning=False):
    from io import BytesIO

    with BytesIO() as f:
        if versioning:
            metadata = json.dumps({"version": version})
            f.write(metadata.encode("utf-8"))
            f.write("\n".encode("utf-8"))
        pickle.dump(record, f)
        content = f.getvalue()

    pkl_open = gzip.open if filename.endswith(".pklz") else open
    tmpfile = filename + ".tmp"
    with pkl_open(tmpfile, "wb") as pkl_file:
        pkl_file.write(content)
    os.rename(tmpfile, filename)


rst_levels = ["=", "-", "~", "+"]


def write_rst_header(header, level=0):
    return "\n".join((header, "".join([rst_levels[level] for _ in header]))) + "\n\n"


def write_rst_list(items, prefix=""):
    out = []
    for item in ensure_list(items):
        out.append("{} {}".format(prefix, str(item)))
    return "\n".join(out) + "\n\n"


def write_rst_dict(info, prefix=""):
    out = []
    for key, value in sorted(info.items()):
        out.append("{}* {} : {}".format(prefix, key, str(value)))
    return "\n".join(out) + "\n\n"


def dist_is_editable(dist):
    """Is distribution an editable install?

    Parameters
    ----------
    dist : string
        Package name

    # Borrowed from `pip`'s' API
    """
    for path_item in sys.path:
        egg_link = op.join(path_item, dist + ".egg-link")
        if op.isfile(egg_link):
            return True
    return False


def emptydirs(path, noexist_ok=False):
    """
    Empty an existing directory, without deleting it. Do not
    raise error if the path does not exist and noexist_ok is True.

    Parameters
    ----------
    path : directory that should be empty

    """
    fmlogger.debug("Removing contents of %s", path)

    if noexist_ok and not op.exists(path):
        return True

    if op.isfile(path):
        raise OSError('path "%s" should be a directory' % path)

    try:
        shutil.rmtree(path)
    except OSError as ex:
        elcont = os.listdir(path)
        if ex.errno == errno.ENOTEMPTY and not elcont:
            fmlogger.warning(
                "An exception was raised trying to remove old %s, but the path"
                " seems empty. Is it an NFS mount?. Passing the exception.",
                path,
            )
        elif ex.errno == errno.ENOTEMPTY and elcont:
            fmlogger.debug("Folder %s contents (%d items).", path, len(elcont))
            raise ex
        else:
            raise ex

    os.makedirs(path)


def silentrm(filename):
    """
    Equivalent to ``rm -f``, returns ``False`` if the file did not
    exist.

    Parameters
    ----------

    filename : str
        file to be deleted

    """
    try:
        os.remove(filename)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise
        return False
    return True


def which(cmd, env=None, pathext=None):
    """
    Return the path to an executable which would be run if the given
    cmd was called. If no cmd would be called, return ``None``.

    Code for Python < 3.3 is based on a code snippet from
    http://orip.org/2009/08/python-checking-if-executable-exists-in.html

    """

    if pathext is None:
        pathext = os.getenv("PATHEXT", "").split(os.pathsep)
        pathext.insert(0, "")

    path = os.getenv("PATH", os.defpath)
    if env and "PATH" in env:
        path = env.get("PATH")

    for ext in pathext:
        filename = shutil.which(cmd + ext, path=path)
        if filename:
            return filename
    return None


def get_dependencies(name, environ):
    """Return library dependencies of a dynamically linked executable

    Uses otool on darwin, ldd on linux. Currently doesn't support windows.

    """
    command = None
    if sys.platform == "darwin":
        command = "otool -L `which %s`" % name
    elif "linux" in sys.platform:
        command = "ldd `which %s`" % name
    else:
        return "Platform %s not supported" % sys.platform

    deps = None
    try:
        proc = sp.Popen(
            command, stdout=sp.PIPE, stderr=sp.PIPE, shell=True, env=environ
        )
        o, e = proc.communicate()
        deps = o.rstrip()
    except Exception as ex:
        deps = f'{command!r} failed'
        fmlogger.warning(f"Could not get dependencies of {name}s. Error:\n{ex}")
    return deps


def canonicalize_env(env):
    """Windows requires that environment be dicts with str as keys and values
    This function converts any unicode entries for Windows only, returning the
    dictionary untouched in other environments.

    Parameters
    ----------
    env : dict
        environment dictionary with unicode or bytes keys and values

    Returns
    -------
    env : dict
        Windows: environment dictionary with str keys and values
        Other: untouched input ``env``
    """
    if os.name != "nt":
        return env

    out_env = {}
    for key, val in env.items():
        if not isinstance(key, str):
            key = key.decode("utf-8")
        if not isinstance(val, str):
            val = val.decode("utf-8")
        out_env[key] = val
    return out_env


def relpath(path, start=None):
    """Return a relative version of a path"""

    try:
        return op.relpath(path, start)
    except AttributeError:
        pass

    if start is None:
        start = os.curdir
    if not path:
        raise ValueError("no path specified")
    start_list = op.abspath(start).split(op.sep)
    path_list = op.abspath(path).split(op.sep)
    if start_list[0].lower() != path_list[0].lower():
        unc_path, rest = op.splitunc(path)
        unc_start, rest = op.splitunc(start)
        if bool(unc_path) ^ bool(unc_start):
            raise ValueError(
                ("Cannot mix UNC and non-UNC paths " "(%s and %s)") % (path, start)
            )
        else:
            raise ValueError(
                "path is on drive %s, start on drive %s" % (path_list[0], start_list[0])
            )
    # Work out how much of the filepath is shared by start and path.
    for i in range(min(len(start_list), len(path_list))):
        if start_list[i].lower() != path_list[i].lower():
            break
    else:
        i += 1

    rel_list = [op.pardir] * (len(start_list) - i) + path_list[i:]
    if not rel_list:
        return os.curdir
    return op.join(*rel_list)


@contextlib.contextmanager
def indirectory(path):
    cwd = os.getcwd()
    os.chdir(str(path))
    try:
        yield
    finally:
        os.chdir(cwd)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Curvature


def test_Curvature_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        averages=dict(
            argstr="-a %d",
        ),
        copy_input=dict(),
        distances=dict(
            argstr="-distances %d %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=True,
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        n=dict(
            argstr="-n",
        ),
        save=dict(
            argstr="-w",
        ),
        subjects_dir=dict(),
        threshold=dict(
            argstr="-thresh %.3f",
        ),
    )
    inputs = Curvature.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Curvature_outputs():
    output_map = dict(
        out_gauss=dict(
            extensions=None,
        ),
        out_mean=dict(
            extensions=None,
        ),
    )
    outputs = Curvature.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class GradientAnisotropicDiffusionInputSpec(CommandLineInputSpec):
    conductance = traits.Float(
        desc="Conductance controls the sensitivity of the conductance term. As a general rule, the lower the value, the more strongly the filter preserves edges. A high value will cause diffusion (smoothing) across edges. Note that the number of iterations controls how much smoothing is done within regions bounded by edges.",
        argstr="--conductance %f",
    )
    iterations = traits.Int(
        desc="The more iterations, the more smoothing. Each iteration takes the same amount of time. If it takes 10 seconds for one iteration, then it will take 100 seconds for 10 iterations. Note that the conductance controls how much each iteration smooths across edges.",
        argstr="--iterations %d",
    )
    timeStep = traits.Float(
        desc="The time step depends on the dimensionality of the image. In Slicer the images are 3D and the default (.0625) time step will provide a stable solution.",
        argstr="--timeStep %f",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be filtered", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class GradientAnisotropicDiffusionOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class GradientAnisotropicDiffusion(SEMLikeCommandLine):
    """title: Gradient Anisotropic Diffusion

category: Filtering.Denoising

description: Runs gradient anisotropic diffusion on a volume.

Anisotropic diffusion methods reduce noise (or unwanted detail) in images while preserving specific image features, like edges.  For many applications, there is an assumption that light-dark transitions (edges) are interesting.  Standard isotropic diffusion methods move and blur light-dark boundaries.  Anisotropic diffusion methods are formulated to specifically preserve edges. The conductance term for this implementation is a function of the gradient magnitude of the image at each point, reducing the strength of diffusion at edges. The numerical implementation of this equation is similar to that described in the Perona-Malik paper, but uses a more robust technique for gradient magnitude estimation and has been generalized to N-dimensions.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/GradientAnisotropicDiffusion

contributor: Bill Lorensen (GE)

acknowledgements: This command module was derived from Insight/Examples (copyright) Insight Software Consortium

"""

    input_spec = GradientAnisotropicDiffusionInputSpec
    output_spec = GradientAnisotropicDiffusionOutputSpec
    _cmd = "GradientAnisotropicDiffusion "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}


class CurvatureAnisotropicDiffusionInputSpec(CommandLineInputSpec):
    conductance = traits.Float(
        desc="Conductance controls the sensitivity of the conductance term. As a general rule, the lower the value, the more strongly the filter preserves edges. A high value will cause diffusion (smoothing) across edges. Note that the number of iterations controls how much smoothing is done within regions bounded by edges.",
        argstr="--conductance %f",
    )
    iterations = traits.Int(
        desc="The more iterations, the more smoothing. Each iteration takes the same amount of time. If it takes 10 seconds for one iteration, then it will take 100 seconds for 10 iterations. Note that the conductance controls how much each iteration smooths across edges.",
        argstr="--iterations %d",
    )
    timeStep = traits.Float(
        desc="The time step depends on the dimensionality of the image. In Slicer the images are 3D and the default (.0625) time step will provide a stable solution.",
        argstr="--timeStep %f",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be filtered", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class CurvatureAnisotropicDiffusionOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class CurvatureAnisotropicDiffusion(SEMLikeCommandLine):
    """title: Curvature Anisotropic Diffusion

category: Filtering.Denoising

description: Performs anisotropic diffusion on an image using a modified curvature diffusion equation (MCDE).

MCDE does not exhibit the edge enhancing properties of classic anisotropic diffusion, which can under certain conditions undergo a 'negative' diffusion, which enhances the contrast of edges.  Equations of the form of MCDE always undergo positive diffusion, with the conductance term only varying the strength of that diffusion.

 Qualitatively, MCDE compares well with other non-linear diffusion techniques.  It is less sensitive to contrast than classic Perona-Malik style diffusion, and preserves finer detailed structures in images.  There is a potential speed trade-off for using this function in place of Gradient Anisotropic Diffusion.  Each iteration of the solution takes roughly twice as long.  Fewer iterations, however, may be required to reach an acceptable solution.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/CurvatureAnisotropicDiffusion

contributor: Bill Lorensen (GE)

acknowledgements: This command module was derived from Insight/Examples (copyright) Insight Software Consortium

"""

    input_spec = CurvatureAnisotropicDiffusionInputSpec
    output_spec = CurvatureAnisotropicDiffusionOutputSpec
    _cmd = "CurvatureAnisotropicDiffusion "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}


class GaussianBlurImageFilterInputSpec(CommandLineInputSpec):
    sigma = traits.Float(
        desc="Sigma value in physical units (e.g., mm) of the Gaussian kernel",
        argstr="--sigma %f",
    )
    inputVolume = File(position=-2, desc="Input volume", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Blurred Volume",
        argstr="%s",
    )


class GaussianBlurImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Blurred Volume", exists=True)


class GaussianBlurImageFilter(SEMLikeCommandLine):
    """title: Gaussian Blur Image Filter

category: Filtering.Denoising

description: Apply a gaussian blurr to an image

version: 0.1.0.$Revision: 1.1 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/GaussianBlurImageFilter

contributor: Julien Jomier (Kitware), Stephen Aylward (Kitware)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = GaussianBlurImageFilterInputSpec
    output_spec = GaussianBlurImageFilterOutputSpec
    _cmd = "GaussianBlurImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}


class MedianImageFilterInputSpec(CommandLineInputSpec):
    neighborhood = InputMultiPath(
        traits.Int,
        desc="The size of the neighborhood in each dimension",
        sep=",",
        argstr="--neighborhood %s",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be filtered", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class MedianImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class MedianImageFilter(SEMLikeCommandLine):
    """title: Median Image Filter

category: Filtering.Denoising

description: The MedianImageFilter is commonly used as a robust approach for noise reduction. This filter is particularly efficient against "salt-and-pepper" noise. In other words, it is robust to the presence of gray-level outliers. MedianImageFilter computes the value of each output pixel as the statistical median of the neighborhood of values around the corresponding input pixel.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/MedianImageFilter

contributor: Bill Lorensen (GE)

acknowledgements: This command module was derived from Insight/Examples/Filtering/MedianImageFilter (copyright) Insight Software Consortium

"""

    input_spec = MedianImageFilterInputSpec
    output_spec = MedianImageFilterOutputSpec
    _cmd = "MedianImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import GLMFit


def test_GLMFit_inputs():
    input_map = dict(
        allow_ill_cond=dict(
            argstr="--illcond",
        ),
        allow_repeated_subjects=dict(
            argstr="--allowsubjrep",
        ),
        args=dict(
            argstr="%s",
        ),
        calc_AR1=dict(
            argstr="--tar1",
        ),
        check_opts=dict(
            argstr="--checkopts",
        ),
        compute_log_y=dict(
            argstr="--logy",
        ),
        contrast=dict(
            argstr="--C %s...",
        ),
        cortex=dict(
            argstr="--cortex",
            xor=["label_file"],
        ),
        debug=dict(
            argstr="--debug",
        ),
        design=dict(
            argstr="--X %s",
            extensions=None,
            xor=("fsgd", "design", "one_sample"),
        ),
        diag=dict(
            argstr="--diag %d",
        ),
        diag_cluster=dict(
            argstr="--diag-cluster",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_fx_dof=dict(
            argstr="--ffxdof %d",
            xor=["fixed_fx_dof_file"],
        ),
        fixed_fx_dof_file=dict(
            argstr="--ffxdofdat %d",
            extensions=None,
            xor=["fixed_fx_dof"],
        ),
        fixed_fx_var=dict(
            argstr="--yffxvar %s",
            extensions=None,
        ),
        force_perm=dict(
            argstr="--perm-force",
        ),
        fsgd=dict(
            argstr="--fsgd %s %s",
            xor=("fsgd", "design", "one_sample"),
        ),
        fwhm=dict(
            argstr="--fwhm %f",
        ),
        glm_dir=dict(
            argstr="--glmdir %s",
            genfile=True,
        ),
        hemi=dict(),
        in_file=dict(
            argstr="--y %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        invert_mask=dict(
            argstr="--mask-inv",
        ),
        label_file=dict(
            argstr="--label %s",
            extensions=None,
            xor=["cortex"],
        ),
        mask_file=dict(
            argstr="--mask %s",
            extensions=None,
        ),
        no_contrast_ok=dict(
            argstr="--no-contrasts-ok",
        ),
        no_est_fwhm=dict(
            argstr="--no-est-fwhm",
        ),
        no_mask_smooth=dict(
            argstr="--no-mask-smooth",
        ),
        no_prune=dict(
            argstr="--no-prune",
            xor=["prunethresh"],
        ),
        one_sample=dict(
            argstr="--osgm",
            xor=("one_sample", "fsgd", "design", "contrast"),
        ),
        pca=dict(
            argstr="--pca",
        ),
        per_voxel_reg=dict(
            argstr="--pvr %s...",
        ),
        profile=dict(
            argstr="--profile %d",
        ),
        prune=dict(
            argstr="--prune",
        ),
        prune_thresh=dict(
            argstr="--prune_thr %f",
            xor=["noprune"],
        ),
        resynth_test=dict(
            argstr="--resynthtest %d",
        ),
        save_cond=dict(
            argstr="--save-cond",
        ),
        save_estimate=dict(
            argstr="--yhat-save",
        ),
        save_res_corr_mtx=dict(
            argstr="--eres-scm",
        ),
        save_residual=dict(
            argstr="--eres-save",
        ),
        seed=dict(
            argstr="--seed %d",
        ),
        self_reg=dict(
            argstr="--selfreg %d %d %d",
        ),
        sim_done_file=dict(
            argstr="--sim-done %s",
            extensions=None,
        ),
        sim_sign=dict(
            argstr="--sim-sign %s",
        ),
        simulation=dict(
            argstr="--sim %s %d %f %s",
        ),
        subject_id=dict(),
        subjects_dir=dict(),
        surf=dict(
            argstr="--surf %s %s %s",
            requires=["subject_id", "hemi"],
        ),
        surf_geo=dict(
            usedefault=True,
        ),
        synth=dict(
            argstr="--synth",
        ),
        uniform=dict(
            argstr="--uniform %f %f",
        ),
        var_fwhm=dict(
            argstr="--var-fwhm %f",
        ),
        vox_dump=dict(
            argstr="--voxdump %d %d %d",
        ),
        weight_file=dict(
            extensions=None,
            xor=["weighted_ls"],
        ),
        weight_inv=dict(
            argstr="--w-inv",
            xor=["weighted_ls"],
        ),
        weight_sqrt=dict(
            argstr="--w-sqrt",
            xor=["weighted_ls"],
        ),
        weighted_ls=dict(
            argstr="--wls %s",
            extensions=None,
            xor=("weight_file", "weight_inv", "weight_sqrt"),
        ),
    )
    inputs = GLMFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_GLMFit_outputs():
    output_map = dict(
        beta_file=dict(
            extensions=None,
        ),
        dof_file=dict(
            extensions=None,
        ),
        error_file=dict(
            extensions=None,
        ),
        error_stddev_file=dict(
            extensions=None,
        ),
        error_var_file=dict(
            extensions=None,
        ),
        estimate_file=dict(
            extensions=None,
        ),
        frame_eigenvectors=dict(
            extensions=None,
        ),
        ftest_file=dict(),
        fwhm_file=dict(
            extensions=None,
        ),
        gamma_file=dict(),
        gamma_var_file=dict(),
        glm_dir=dict(),
        mask_file=dict(
            extensions=None,
        ),
        sig_file=dict(),
        singular_values=dict(
            extensions=None,
        ),
        spatial_eigenvectors=dict(
            extensions=None,
        ),
        svd_stats_file=dict(
            extensions=None,
        ),
    )
    outputs = GLMFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..maths import PercentileImage


def test_PercentileImage_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        dimension=dict(
            argstr="-%sperc",
            position=4,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=2,
        ),
        internal_datatype=dict(
            argstr="-dt %s",
            position=1,
        ),
        nan2zeros=dict(
            argstr="-nan",
            position=3,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            position=-2,
        ),
        output_datatype=dict(
            argstr="-odt %s",
            position=-1,
        ),
        output_type=dict(),
        perc=dict(
            argstr="%f",
            position=5,
        ),
    )
    inputs = PercentileImage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_PercentileImage_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = PercentileImage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..developer import MedicAlgorithmSPECTRE2010


def test_MedicAlgorithmSPECTRE2010_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inApply=dict(
            argstr="--inApply %s",
        ),
        inAtlas=dict(
            argstr="--inAtlas %s",
            extensions=None,
        ),
        inBackground=dict(
            argstr="--inBackground %f",
        ),
        inCoarse=dict(
            argstr="--inCoarse %f",
        ),
        inCost=dict(
            argstr="--inCost %s",
        ),
        inDegrees=dict(
            argstr="--inDegrees %s",
        ),
        inFind=dict(
            argstr="--inFind %s",
        ),
        inFine=dict(
            argstr="--inFine %f",
        ),
        inImage=dict(
            argstr="--inImage %s",
        ),
        inInhomogeneity=dict(
            argstr="--inInhomogeneity %s",
        ),
        inInitial=dict(
            argstr="--inInitial %d",
        ),
        inInitial2=dict(
            argstr="--inInitial2 %f",
        ),
        inInput=dict(
            argstr="--inInput %s",
            extensions=None,
        ),
        inMMC=dict(
            argstr="--inMMC %d",
        ),
        inMMC2=dict(
            argstr="--inMMC2 %d",
        ),
        inMaximum=dict(
            argstr="--inMaximum %f",
        ),
        inMinimum=dict(
            argstr="--inMinimum %f",
        ),
        inMinimum2=dict(
            argstr="--inMinimum2 %f",
        ),
        inMultiple=dict(
            argstr="--inMultiple %d",
        ),
        inMultithreading=dict(
            argstr="--inMultithreading %s",
        ),
        inNumber=dict(
            argstr="--inNumber %d",
        ),
        inNumber2=dict(
            argstr="--inNumber2 %d",
        ),
        inOutput=dict(
            argstr="--inOutput %s",
        ),
        inOutput2=dict(
            argstr="--inOutput2 %s",
        ),
        inOutput3=dict(
            argstr="--inOutput3 %s",
        ),
        inOutput4=dict(
            argstr="--inOutput4 %s",
        ),
        inOutput5=dict(
            argstr="--inOutput5 %s",
        ),
        inRegistration=dict(
            argstr="--inRegistration %s",
        ),
        inResample=dict(
            argstr="--inResample %s",
        ),
        inRun=dict(
            argstr="--inRun %s",
        ),
        inSkip=dict(
            argstr="--inSkip %s",
        ),
        inSmoothing=dict(
            argstr="--inSmoothing %f",
        ),
        inSubsample=dict(
            argstr="--inSubsample %s",
        ),
        inUse=dict(
            argstr="--inUse %s",
        ),
        null=dict(
            argstr="--null %s",
        ),
        outFANTASM=dict(
            argstr="--outFANTASM %s",
            hash_files=False,
        ),
        outMask=dict(
            argstr="--outMask %s",
            hash_files=False,
        ),
        outMidsagittal=dict(
            argstr="--outMidsagittal %s",
            hash_files=False,
        ),
        outOriginal=dict(
            argstr="--outOriginal %s",
            hash_files=False,
        ),
        outPrior=dict(
            argstr="--outPrior %s",
            hash_files=False,
        ),
        outSegmentation=dict(
            argstr="--outSegmentation %s",
            hash_files=False,
        ),
        outSplitHalves=dict(
            argstr="--outSplitHalves %s",
            hash_files=False,
        ),
        outStripped=dict(
            argstr="--outStripped %s",
            hash_files=False,
        ),
        outd0=dict(
            argstr="--outd0 %s",
            hash_files=False,
        ),
        xDefaultMem=dict(
            argstr="-xDefaultMem %d",
        ),
        xMaxProcess=dict(
            argstr="-xMaxProcess %d",
            usedefault=True,
        ),
        xPrefExt=dict(
            argstr="--xPrefExt %s",
        ),
    )
    inputs = MedicAlgorithmSPECTRE2010.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MedicAlgorithmSPECTRE2010_outputs():
    output_map = dict(
        outFANTASM=dict(
            extensions=None,
        ),
        outMask=dict(
            extensions=None,
        ),
        outMidsagittal=dict(
            extensions=None,
        ),
        outOriginal=dict(
            extensions=None,
        ),
        outPrior=dict(
            extensions=None,
        ),
        outSegmentation=dict(
            extensions=None,
        ),
        outSplitHalves=dict(
            extensions=None,
        ),
        outStripped=dict(
            extensions=None,
        ),
        outd0=dict(
            extensions=None,
        ),
    )
    outputs = MedicAlgorithmSPECTRE2010.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import Math


def test_Math_inputs():
    input_map = dict(
        abs=dict(
            argstr="-abs",
        ),
        args=dict(
            argstr="%s",
        ),
        calc_add=dict(
            argstr="-add",
        ),
        calc_and=dict(
            argstr="-and",
        ),
        calc_div=dict(
            argstr="-div",
        ),
        calc_mul=dict(
            argstr="-mult",
        ),
        calc_not=dict(
            argstr="-not",
        ),
        calc_or=dict(
            argstr="-or",
        ),
        calc_sub=dict(
            argstr="-sub",
        ),
        check_dimensions=dict(
            argstr="-check_dimensions",
            xor=("check_dimensions", "no_check_dimensions"),
        ),
        clamp=dict(
            argstr="-clamp -const2 %s %s",
        ),
        clobber=dict(
            argstr="-clobber",
            usedefault=True,
        ),
        copy_header=dict(
            argstr="-copy_header",
            xor=("copy_header", "no_copy_header"),
        ),
        count_valid=dict(
            argstr="-count_valid",
        ),
        dimension=dict(
            argstr="-dimension %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exp=dict(
            argstr="-exp -const2 %s %s",
        ),
        filelist=dict(
            argstr="-filelist %s",
            extensions=None,
            mandatory=True,
            xor=("input_files", "filelist"),
        ),
        format_byte=dict(
            argstr="-byte",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_double=dict(
            argstr="-double",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_filetype=dict(
            argstr="-filetype",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_float=dict(
            argstr="-float",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_int=dict(
            argstr="-int",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_long=dict(
            argstr="-long",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_short=dict(
            argstr="-short",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_signed=dict(
            argstr="-signed",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_unsigned=dict(
            argstr="-unsigned",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        ignore_nan=dict(
            argstr="-ignore_nan",
        ),
        input_files=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
            sep=" ",
            xor=("input_files", "filelist"),
        ),
        invert=dict(
            argstr="-invert -const %s",
        ),
        isnan=dict(
            argstr="-isnan",
        ),
        log=dict(
            argstr="-log -const2 %s %s",
        ),
        max_buffer_size_in_kb=dict(
            argstr="-max_buffer_size_in_kb %d",
            usedefault=True,
        ),
        maximum=dict(
            argstr="-maximum",
        ),
        minimum=dict(
            argstr="-minimum",
        ),
        nisnan=dict(
            argstr="-nisnan",
        ),
        no_check_dimensions=dict(
            argstr="-nocheck_dimensions",
            xor=("check_dimensions", "no_check_dimensions"),
        ),
        no_copy_header=dict(
            argstr="-nocopy_header",
            xor=("copy_header", "no_copy_header"),
        ),
        nsegment=dict(
            argstr="-nsegment -const2 %s %s",
        ),
        output_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=["input_files"],
            name_template="%s_mincmath.mnc",
            position=-1,
        ),
        output_illegal=dict(
            argstr="-illegal_value",
            xor=("output_nan", "output_zero", "output_illegal_value"),
        ),
        output_nan=dict(
            argstr="-nan",
            xor=("output_nan", "output_zero", "output_illegal_value"),
        ),
        output_zero=dict(
            argstr="-zero",
            xor=("output_nan", "output_zero", "output_illegal_value"),
        ),
        percentdiff=dict(
            argstr="-percentdiff",
        ),
        propagate_nan=dict(
            argstr="-propagate_nan",
        ),
        scale=dict(
            argstr="-scale -const2 %s %s",
        ),
        segment=dict(
            argstr="-segment -const2 %s %s",
        ),
        sqrt=dict(
            argstr="-sqrt",
        ),
        square=dict(
            argstr="-square",
        ),
        test_eq=dict(
            argstr="-eq",
        ),
        test_ge=dict(
            argstr="-ge",
        ),
        test_gt=dict(
            argstr="-gt",
        ),
        test_le=dict(
            argstr="-le",
        ),
        test_lt=dict(
            argstr="-lt",
        ),
        test_ne=dict(
            argstr="-ne",
        ),
        two=dict(
            argstr="-2",
        ),
        voxel_range=dict(
            argstr="-range %d %d",
        ),
    )
    inputs = Math.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Math_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = Math.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import SliceTimer


def test_SliceTimer_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        custom_order=dict(
            argstr="--ocustom=%s",
            extensions=None,
        ),
        custom_timings=dict(
            argstr="--tcustom=%s",
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        global_shift=dict(
            argstr="--tglobal",
        ),
        in_file=dict(
            argstr="--in=%s",
            extensions=None,
            mandatory=True,
            position=0,
        ),
        index_dir=dict(
            argstr="--down",
        ),
        interleaved=dict(
            argstr="--odd",
        ),
        out_file=dict(
            argstr="--out=%s",
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        slice_direction=dict(
            argstr="--direction=%d",
        ),
        time_repetition=dict(
            argstr="--repeat=%f",
        ),
    )
    inputs = SliceTimer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SliceTimer_outputs():
    output_map = dict(
        slice_time_corrected_file=dict(
            extensions=None,
        ),
    )
    outputs = SliceTimer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Tkregister2


def test_Tkregister2_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fsl_in_matrix=dict(
            argstr="--fsl %s",
            extensions=None,
        ),
        fsl_out=dict(
            argstr="--fslregout %s",
        ),
        fstal=dict(
            argstr="--fstal",
            xor=["target_image", "moving_image", "reg_file"],
        ),
        fstarg=dict(
            argstr="--fstarg",
            xor=["target_image"],
        ),
        invert_lta_in=dict(
            requires=["lta_in"],
        ),
        invert_lta_out=dict(
            argstr="--ltaout-inv",
            requires=["lta_in"],
        ),
        lta_in=dict(
            argstr="--lta %s",
            extensions=None,
        ),
        lta_out=dict(
            argstr="--ltaout %s",
        ),
        moving_image=dict(
            argstr="--mov %s",
            extensions=None,
            mandatory=True,
        ),
        movscale=dict(
            argstr="--movscale %f",
        ),
        noedit=dict(
            argstr="--noedit",
            usedefault=True,
        ),
        reg_file=dict(
            argstr="--reg %s",
            extensions=None,
            mandatory=True,
            usedefault=True,
        ),
        reg_header=dict(
            argstr="--regheader",
        ),
        subject_id=dict(
            argstr="--s %s",
        ),
        subjects_dir=dict(),
        target_image=dict(
            argstr="--targ %s",
            extensions=None,
            xor=["fstarg"],
        ),
        xfm=dict(
            argstr="--xfm %s",
            extensions=None,
        ),
    )
    inputs = Tkregister2.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Tkregister2_outputs():
    output_map = dict(
        fsl_file=dict(
            extensions=None,
        ),
        lta_file=dict(
            extensions=None,
        ),
        reg_file=dict(
            extensions=None,
        ),
    )
    outputs = Tkregister2.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class OtsuThresholdSegmentationInputSpec(CommandLineInputSpec):
    brightObjects = traits.Bool(
        desc="Segmenting bright objects on a dark background or dark objects on a bright background.",
        argstr="--brightObjects ",
    )
    numberOfBins = traits.Int(
        desc="This is an advanced parameter. The number of bins in the histogram used to model the probability mass function of the two intensity distributions. Small numbers of bins may result in a more conservative threshold. The default should suffice for most applications. Experimentation is the only way to see the effect of varying this parameter.",
        argstr="--numberOfBins %d",
    )
    faceConnected = traits.Bool(
        desc="This is an advanced parameter. Adjacent voxels are face connected. This affects the connected component algorithm. If this parameter is false, more regions are likely to be identified.",
        argstr="--faceConnected ",
    )
    minimumObjectSize = traits.Int(
        desc="Minimum size of object to retain. This parameter can be used to get rid of small regions in noisy images.",
        argstr="--minimumObjectSize %d",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be segmented", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class OtsuThresholdSegmentationOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class OtsuThresholdSegmentation(SEMLikeCommandLine):
    """title: Otsu Threshold Segmentation

category: Legacy.Segmentation

description: This filter creates a labeled image from a grayscale image. First, it calculates an optimal threshold that separates the image into foreground and background. This threshold separates those two classes so that their intra-class variance is minimal (see http://en.wikipedia.org/wiki/Otsu%27s_method). Then the filter runs a connected component algorithm to generate unique labels for each connected region of the foreground. Finally, the resulting image is relabeled to provide consecutive numbering.

version: 1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/OtsuThresholdSegmentation

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = OtsuThresholdSegmentationInputSpec
    output_spec = OtsuThresholdSegmentationOutputSpec
    _cmd = "OtsuThresholdSegmentation "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class ResampleScalarVectorDWIVolumeInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-2, desc="Input Volume to be resampled", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Resampled Volume",
        argstr="%s",
    )
    Reference = File(
        desc="Reference Volume (spacing,size,orientation,origin)",
        exists=True,
        argstr="--Reference %s",
    )
    transformationFile = File(exists=True, argstr="--transformationFile %s")
    defField = File(
        desc="File containing the deformation field (3D vector image containing vectors with 3 components)",
        exists=True,
        argstr="--defField %s",
    )
    hfieldtype = traits.Enum(
        "displacement",
        "h-Field",
        desc="Set if the deformation field is an h-Field",
        argstr="--hfieldtype %s",
    )
    interpolation = traits.Enum(
        "linear",
        "nn",
        "ws",
        "bs",
        desc="Sampling algorithm (linear or nn (nearest neighborhoor), ws (WindowedSinc), bs (BSpline) )",
        argstr="--interpolation %s",
    )
    transform_order = traits.Enum(
        "input-to-output",
        "output-to-input",
        desc="Select in what order the transforms are read",
        argstr="--transform_order %s",
    )
    notbulk = traits.Bool(
        desc="The transform following the BSpline transform is not set as a bulk transform for the BSpline transform",
        argstr="--notbulk ",
    )
    spaceChange = traits.Bool(
        desc="Space Orientation between transform and image is different (RAS/LPS) (warning: if the transform is a Transform Node in Slicer3, do not select)",
        argstr="--spaceChange ",
    )
    rotation_point = traits.List(
        desc="Rotation Point in case of rotation around a point (otherwise useless)",
        argstr="--rotation_point %s",
    )
    centered_transform = traits.Bool(
        desc="Set the center of the transformation to the center of the input image",
        argstr="--centered_transform ",
    )
    image_center = traits.Enum(
        "input",
        "output",
        desc="Image to use to center the transform (used only if 'Centered Transform' is selected)",
        argstr="--image_center %s",
    )
    Inverse_ITK_Transformation = traits.Bool(
        desc="Inverse the transformation before applying it from output image to input image",
        argstr="--Inverse_ITK_Transformation ",
    )
    spacing = InputMultiPath(
        traits.Float,
        desc="Spacing along each dimension (0 means use input spacing)",
        sep=",",
        argstr="--spacing %s",
    )
    size = InputMultiPath(
        traits.Float,
        desc="Size along each dimension (0 means use input size)",
        sep=",",
        argstr="--size %s",
    )
    origin = traits.List(desc="Origin of the output Image", argstr="--origin %s")
    direction_matrix = InputMultiPath(
        traits.Float,
        desc="9 parameters of the direction matrix by rows (ijk to LPS if LPS transform, ijk to RAS if RAS transform)",
        sep=",",
        argstr="--direction_matrix %s",
    )
    number_of_thread = traits.Int(
        desc="Number of thread used to compute the output image",
        argstr="--number_of_thread %d",
    )
    default_pixel_value = traits.Float(
        desc="Default pixel value for samples falling outside of the input region",
        argstr="--default_pixel_value %f",
    )
    window_function = traits.Enum(
        "h",
        "c",
        "w",
        "l",
        "b",
        desc="Window Function , h = Hamming , c = Cosine , w = Welch , l = Lanczos , b = Blackman",
        argstr="--window_function %s",
    )
    spline_order = traits.Int(desc="Spline Order", argstr="--spline_order %d")
    transform_matrix = InputMultiPath(
        traits.Float,
        desc="12 parameters of the transform matrix by rows ( --last 3 being translation-- )",
        sep=",",
        argstr="--transform_matrix %s",
    )
    transform = traits.Enum(
        "rt",
        "a",
        desc="Transform algorithm, rt = Rigid Transform, a = Affine Transform",
        argstr="--transform %s",
    )


class ResampleScalarVectorDWIVolumeOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Resampled Volume", exists=True)


class ResampleScalarVectorDWIVolume(SEMLikeCommandLine):
    """title: Resample Scalar/Vector/DWI Volume

category: Filtering

description: This module implements image and vector-image resampling through  the use of itk Transforms.It can also handle diffusion weighted MRI image resampling. "Resampling" is performed in space coordinates, not pixel/grid coordinates. It is quite important to ensure that image spacing is properly set on the images involved. The interpolator is required since the mapping from one space to the other will often require evaluation of the intensity of the image at non-grid positions.

Warning: To resample DWMR Images, use nrrd input and output files.

Warning: Do not use to resample Diffusion Tensor Images, tensors would  not be reoriented

version: 0.1

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ResampleScalarVectorDWIVolume

contributor: Francois Budin (UNC)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149. Information on the National Centers for Biomedical Computing can be obtained from http://nihroadmap.nih.gov/bioinformatics

"""

    input_spec = ResampleScalarVectorDWIVolumeInputSpec
    output_spec = ResampleScalarVectorDWIVolumeOutputSpec
    _cmd = "ResampleScalarVectorDWIVolume "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dcm2nii import Dcm2nii


def test_Dcm2nii_inputs():
    input_map = dict(
        anonymize=dict(
            argstr="-a",
            usedefault=True,
        ),
        args=dict(
            argstr="%s",
        ),
        collapse_folders=dict(
            argstr="-c",
            usedefault=True,
        ),
        config_file=dict(
            argstr="-b %s",
            extensions=None,
            genfile=True,
        ),
        convert_all_pars=dict(
            argstr="-v",
            usedefault=True,
        ),
        date_in_filename=dict(
            argstr="-d",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        events_in_filename=dict(
            argstr="-e",
            usedefault=True,
        ),
        gzip_output=dict(
            argstr="-g",
            usedefault=True,
        ),
        id_in_filename=dict(
            argstr="-i",
            usedefault=True,
        ),
        nii_output=dict(
            argstr="-n",
            usedefault=True,
        ),
        output_dir=dict(
            argstr="-o %s",
            genfile=True,
        ),
        protocol_in_filename=dict(
            argstr="-p",
            usedefault=True,
        ),
        reorient=dict(
            argstr="-r",
        ),
        reorient_and_crop=dict(
            argstr="-x",
            usedefault=True,
        ),
        source_dir=dict(
            argstr="%s",
            mandatory=True,
            position=-1,
            xor=["source_names"],
        ),
        source_in_filename=dict(
            argstr="-f",
            usedefault=True,
        ),
        source_names=dict(
            argstr="%s",
            copyfile=False,
            mandatory=True,
            position=-1,
            xor=["source_dir"],
        ),
        spm_analyze=dict(
            argstr="-s",
            xor=["nii_output"],
        ),
    )
    inputs = Dcm2nii.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Dcm2nii_outputs():
    output_map = dict(
        bvals=dict(),
        bvecs=dict(),
        converted_files=dict(),
        reoriented_and_cropped_files=dict(),
        reoriented_files=dict(),
    )
    outputs = Dcm2nii.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-

import os
import os.path as op
import datetime
import string
import networkx as nx

from ...utils.filemanip import split_filename
from ..base import (
    BaseInterfaceInputSpec,
    traits,
    File,
    TraitedSpec,
    InputMultiPath,
    isdefined,
)
from .base import CFFBaseInterface, have_cfflib


class CFFConverterInputSpec(BaseInterfaceInputSpec):
    graphml_networks = InputMultiPath(
        File(exists=True), desc="list of graphML networks"
    )
    gpickled_networks = InputMultiPath(
        File(exists=True), desc="list of gpickled Networkx graphs"
    )

    gifti_surfaces = InputMultiPath(File(exists=True), desc="list of GIFTI surfaces")
    gifti_labels = InputMultiPath(File(exists=True), desc="list of GIFTI labels")
    nifti_volumes = InputMultiPath(File(exists=True), desc="list of NIFTI volumes")
    tract_files = InputMultiPath(File(exists=True), desc="list of Trackvis fiber files")

    timeseries_files = InputMultiPath(
        File(exists=True), desc="list of HDF5 timeseries files"
    )
    script_files = InputMultiPath(
        File(exists=True), desc="list of script files to include"
    )
    data_files = InputMultiPath(
        File(exists=True), desc="list of external data files (i.e. Numpy, HD5, XML) "
    )

    title = traits.Str(desc="Connectome Title")
    creator = traits.Str(desc="Creator")
    email = traits.Str(desc="Email address")
    publisher = traits.Str(desc="Publisher")
    license = traits.Str(desc="License")
    rights = traits.Str(desc="Rights")
    references = traits.Str(desc="References")
    relation = traits.Str(desc="Relation")
    species = traits.Str("Homo sapiens", desc="Species", usedefault=True)
    description = traits.Str(
        "Created with the Nipype CFF converter", desc="Description", usedefault=True
    )

    out_file = File("connectome.cff", usedefault=True, desc="Output connectome file")


class CFFConverterOutputSpec(TraitedSpec):
    connectome_file = File(exists=True, desc="Output connectome file")


class CFFConverter(CFFBaseInterface):
    """
    Creates a Connectome File Format (CFF) file from input networks, surfaces, volumes, tracts, etcetera....

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> cvt = cmtk.CFFConverter()
    >>> cvt.inputs.title = 'subject 1'
    >>> cvt.inputs.gifti_surfaces = ['lh.pial_converted.gii', 'rh.pial_converted.gii']
    >>> cvt.inputs.tract_files = ['streamlines.trk']
    >>> cvt.inputs.gpickled_networks = ['network0.gpickle']
    >>> cvt.run()                 # doctest: +SKIP
    """

    input_spec = CFFConverterInputSpec
    output_spec = CFFConverterOutputSpec

    def _run_interface(self, runtime):
        import cfflib as cf

        a = cf.connectome()

        if isdefined(self.inputs.title):
            a.connectome_meta.set_title(self.inputs.title)
        else:
            a.connectome_meta.set_title(self.inputs.out_file)

        if isdefined(self.inputs.creator):
            a.connectome_meta.set_creator(self.inputs.creator)
        else:
            # Probably only works on some OSes...
            a.connectome_meta.set_creator(os.getenv("USER"))

        if isdefined(self.inputs.email):
            a.connectome_meta.set_email(self.inputs.email)

        if isdefined(self.inputs.publisher):
            a.connectome_meta.set_publisher(self.inputs.publisher)

        if isdefined(self.inputs.license):
            a.connectome_meta.set_license(self.inputs.license)

        if isdefined(self.inputs.rights):
            a.connectome_meta.set_rights(self.inputs.rights)

        if isdefined(self.inputs.references):
            a.connectome_meta.set_references(self.inputs.references)

        if isdefined(self.inputs.relation):
            a.connectome_meta.set_relation(self.inputs.relation)

        if isdefined(self.inputs.species):
            a.connectome_meta.set_species(self.inputs.species)

        if isdefined(self.inputs.description):
            a.connectome_meta.set_description(self.inputs.description)

        a.connectome_meta.set_created(datetime.date.today())

        count = 0
        if isdefined(self.inputs.graphml_networks):
            for ntwk in self.inputs.graphml_networks:
                # There must be a better way to deal with the unique name problem
                # (i.e. tracks and networks can't use the same name, and previously we were pulling them both from the input files)
                ntwk_name = "Network {cnt}".format(cnt=count)
                a.add_connectome_network_from_graphml(ntwk_name, ntwk)
                count += 1

        if isdefined(self.inputs.gpickled_networks):
            unpickled = []
            for ntwk in self.inputs.gpickled_networks:
                _, ntwk_name, _ = split_filename(ntwk)
                unpickled = nx.read_gpickle(ntwk)
                cnet = cf.CNetwork(name=ntwk_name)
                cnet.set_with_nxgraph(unpickled)
                a.add_connectome_network(cnet)
                count += 1

        count = 0
        if isdefined(self.inputs.tract_files):
            for trk in self.inputs.tract_files:
                _, trk_name, _ = split_filename(trk)
                ctrack = cf.CTrack(trk_name, trk)
                a.add_connectome_track(ctrack)
                count += 1

        count = 0
        if isdefined(self.inputs.gifti_surfaces):
            for surf in self.inputs.gifti_surfaces:
                _, surf_name, _ = split_filename(surf)
                csurf = cf.CSurface.create_from_gifti(
                    "Surface %d - %s" % (count, surf_name), surf
                )
                csurf.fileformat = "Gifti"
                csurf.dtype = "Surfaceset"
                a.add_connectome_surface(csurf)
                count += 1

        count = 0
        if isdefined(self.inputs.gifti_labels):
            for label in self.inputs.gifti_labels:
                _, label_name, _ = split_filename(label)
                csurf = cf.CSurface.create_from_gifti(
                    "Surface Label %d - %s" % (count, label_name), label
                )
                csurf.fileformat = "Gifti"
                csurf.dtype = "Labels"
                a.add_connectome_surface(csurf)
                count += 1

        if isdefined(self.inputs.nifti_volumes):
            for vol in self.inputs.nifti_volumes:
                _, vol_name, _ = split_filename(vol)
                cvol = cf.CVolume.create_from_nifti(vol_name, vol)
                a.add_connectome_volume(cvol)

        if isdefined(self.inputs.script_files):
            for script in self.inputs.script_files:
                _, script_name, _ = split_filename(script)
                cscript = cf.CScript.create_from_file(script_name, script)
                a.add_connectome_script(cscript)

        if isdefined(self.inputs.data_files):
            for data in self.inputs.data_files:
                _, data_name, _ = split_filename(data)
                cda = cf.CData(name=data_name, src=data, fileformat="NumPy")
                if not string.find(data_name, "lengths") == -1:
                    cda.dtype = "FinalFiberLengthArray"
                if not string.find(data_name, "endpoints") == -1:
                    cda.dtype = "FiberEndpoints"
                if not string.find(data_name, "labels") == -1:
                    cda.dtype = "FinalFiberLabels"
                a.add_connectome_data(cda)

        a.print_summary()
        _, name, ext = split_filename(self.inputs.out_file)
        if not ext == ".cff":
            ext = ".cff"
        cf.save_to_cff(a, op.abspath(name + ext))

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        _, name, ext = split_filename(self.inputs.out_file)
        if not ext == ".cff":
            ext = ".cff"
        outputs["connectome_file"] = op.abspath(name + ext)
        return outputs


class MergeCNetworksInputSpec(BaseInterfaceInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc="List of CFF files to extract networks from",
    )
    out_file = File(
        "merged_network_connectome.cff",
        usedefault=True,
        desc="Output CFF file with all the networks added",
    )


class MergeCNetworksOutputSpec(TraitedSpec):
    connectome_file = File(
        exists=True, desc="Output CFF file with all the networks added"
    )


class MergeCNetworks(CFFBaseInterface):
    """ Merges networks from multiple CFF files into one new CFF file.

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> mrg = cmtk.MergeCNetworks()
    >>> mrg.inputs.in_files = ['subj1.cff','subj2.cff']
    >>> mrg.run()                  # doctest: +SKIP

    """

    input_spec = MergeCNetworksInputSpec
    output_spec = MergeCNetworksOutputSpec

    def _run_interface(self, runtime):
        import cfflib as cf

        extracted_networks = []

        for i, con in enumerate(self.inputs.in_files):
            mycon = cf.load(con)
            nets = mycon.get_connectome_network()
            for ne in nets:
                # here, you might want to skip networks with a given
                # metadata information
                ne.load()
                contitle = mycon.get_connectome_meta().get_title()
                ne.set_name(str(i) + ": " + contitle + " - " + ne.get_name())
                ne.set_src(ne.get_name())
                extracted_networks.append(ne)

        # Add networks to new connectome
        newcon = cf.connectome(
            title="All CNetworks", connectome_network=extracted_networks
        )
        # Setting additional metadata
        metadata = newcon.get_connectome_meta()
        metadata.set_creator("My Name")
        metadata.set_email("My Email")

        _, name, ext = split_filename(self.inputs.out_file)
        if not ext == ".cff":
            ext = ".cff"
        cf.save_to_cff(newcon, op.abspath(name + ext))

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        _, name, ext = split_filename(self.inputs.out_file)
        if not ext == ".cff":
            ext = ".cff"
        outputs["connectome_file"] = op.abspath(name + ext)
        return outputs



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import os.path as op

import numpy as np
import networkx as nx

from ... import logging
from ..base import (
    LibraryBaseInterface,
    BaseInterfaceInputSpec,
    traits,
    File,
    TraitedSpec,
    InputMultiPath,
    OutputMultiPath,
    isdefined,
)
from .base import have_cv

iflogger = logging.getLogger("nipype.interface")


def ntwks_to_matrices(in_files, edge_key):
    first = nx.read_gpickle(in_files[0])
    files = len(in_files)
    nodes = len(first.nodes())
    matrix = np.zeros((nodes, nodes, files))
    for idx, name in enumerate(in_files):
        graph = nx.read_gpickle(name)
        for u, v, d in graph.edges(data=True):
            try:
                graph[u][v]["weight"] = d[
                    edge_key
                ]  # Setting the edge requested edge value as weight value
            except:
                raise KeyError(
                    "the graph edges do not have {} attribute".format(edge_key)
                )
        matrix[:, :, idx] = nx.to_numpy_matrix(graph)  # Retrieve the matrix
    return matrix


class NetworkBasedStatisticInputSpec(BaseInterfaceInputSpec):
    in_group1 = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc="Networks for the first group of subjects",
    )
    in_group2 = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc="Networks for the second group of subjects",
    )
    node_position_network = File(
        desc="An optional network used to position the nodes for the output networks"
    )
    number_of_permutations = traits.Int(
        1000, usedefault=True, desc="Number of permutations to perform"
    )
    threshold = traits.Float(3, usedefault=True, desc="T-statistic threshold")
    t_tail = traits.Enum(
        "left",
        "right",
        "both",
        usedefault=True,
        desc='Can be one of "left", "right", or "both"',
    )
    edge_key = traits.Str(
        "number_of_fibers",
        usedefault=True,
        desc='Usually "number_of_fibers, "fiber_length_mean", "fiber_length_std" for matrices made with CMTK'
        'Sometimes "weight" or "value" for functional networks.',
    )
    out_nbs_network = File(desc="Output network with edges identified by the NBS")
    out_nbs_pval_network = File(
        desc="Output network with p-values to weight the edges identified by the NBS"
    )


class NetworkBasedStatisticOutputSpec(TraitedSpec):
    nbs_network = File(
        exists=True, desc="Output network with edges identified by the NBS"
    )
    nbs_pval_network = File(
        exists=True,
        desc="Output network with p-values to weight the edges identified by the NBS",
    )
    network_files = OutputMultiPath(
        File(exists=True), desc="Output network with edges identified by the NBS"
    )


class NetworkBasedStatistic(LibraryBaseInterface):
    """
    Calculates and outputs the average network given a set of input NetworkX gpickle files

    See Also
    --------
    For documentation of Network-based statistic parameters:
    https://github.com/LTS5/connectomeviewer/blob/master/cviewer/libs/pyconto/groupstatistics/nbs/_nbs.py

    Example
    -------
    >>> import nipype.interfaces.cmtk as cmtk
    >>> nbs = cmtk.NetworkBasedStatistic()
    >>> nbs.inputs.in_group1 = ['subj1.pck', 'subj2.pck'] # doctest: +SKIP
    >>> nbs.inputs.in_group2 = ['pat1.pck', 'pat2.pck'] # doctest: +SKIP
    >>> nbs.run()                 # doctest: +SKIP

    """

    input_spec = NetworkBasedStatisticInputSpec
    output_spec = NetworkBasedStatisticOutputSpec
    _pkg = "cviewer"

    def _run_interface(self, runtime):
        from cviewer.libs.pyconto.groupstatistics import nbs

        THRESH = self.inputs.threshold
        K = self.inputs.number_of_permutations
        TAIL = self.inputs.t_tail
        edge_key = self.inputs.edge_key
        details = (
            edge_key
            + "-thresh-"
            + str(THRESH)
            + "-k-"
            + str(K)
            + "-tail-"
            + TAIL
            + ".pck"
        )

        # Fill in the data from the networks
        X = ntwks_to_matrices(self.inputs.in_group1, edge_key)
        Y = ntwks_to_matrices(self.inputs.in_group2, edge_key)

        PVAL, ADJ, _ = nbs.compute_nbs(X, Y, THRESH, K, TAIL)

        iflogger.info("p-values:")
        iflogger.info(PVAL)

        pADJ = ADJ.copy()
        for idx, _ in enumerate(PVAL):
            x, y = np.where(ADJ == idx + 1)
            pADJ[x, y] = PVAL[idx]

        # Create networkx graphs from the adjacency matrix
        nbsgraph = nx.from_numpy_matrix(ADJ)
        nbs_pval_graph = nx.from_numpy_matrix(pADJ)

        # Relabel nodes because they should not start at zero for our convention
        nbsgraph = nx.relabel_nodes(nbsgraph, lambda x: x + 1)
        nbs_pval_graph = nx.relabel_nodes(nbs_pval_graph, lambda x: x + 1)

        if isdefined(self.inputs.node_position_network):
            node_ntwk_name = self.inputs.node_position_network
        else:
            node_ntwk_name = self.inputs.in_group1[0]

        node_network = nx.read_gpickle(node_ntwk_name)
        iflogger.info(
            "Populating node dictionaries with attributes from %s", node_ntwk_name
        )

        for nid, ndata in node_network.nodes(data=True):
            nbsgraph.nodes[nid] = ndata
            nbs_pval_graph.nodes[nid] = ndata

        path = op.abspath("NBS_Result_" + details)
        iflogger.info(path)
        nx.write_gpickle(nbsgraph, path)
        iflogger.info("Saving output NBS edge network as %s", path)

        pval_path = op.abspath("NBS_P_vals_" + details)
        iflogger.info(pval_path)
        nx.write_gpickle(nbs_pval_graph, pval_path)
        iflogger.info("Saving output p-value network as %s", pval_path)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()

        THRESH = self.inputs.threshold
        K = self.inputs.number_of_permutations
        TAIL = self.inputs.t_tail
        edge_key = self.inputs.edge_key
        details = (
            edge_key
            + "-thresh-"
            + str(THRESH)
            + "-k-"
            + str(K)
            + "-tail-"
            + TAIL
            + ".pck"
        )
        path = op.abspath("NBS_Result_" + details)
        pval_path = op.abspath("NBS_P_vals_" + details)

        outputs["nbs_network"] = path
        outputs["nbs_pval_network"] = pval_path
        outputs["network_files"] = [path, pval_path]
        return outputs

    def _gen_outfilename(self, name, ext):
        return name + "." + ext



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import FAST


def test_FAST_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        bias_iters=dict(
            argstr="-I %d",
        ),
        bias_lowpass=dict(
            argstr="-l %d",
            units="mm",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hyper=dict(
            argstr="-H %.2f",
        ),
        img_type=dict(
            argstr="-t %d",
        ),
        in_files=dict(
            argstr="%s",
            copyfile=False,
            mandatory=True,
            position=-1,
        ),
        init_seg_smooth=dict(
            argstr="-f %.3f",
        ),
        init_transform=dict(
            argstr="-a %s",
            extensions=None,
        ),
        iters_afterbias=dict(
            argstr="-O %d",
        ),
        manual_seg=dict(
            argstr="-s %s",
            extensions=None,
        ),
        mixel_smooth=dict(
            argstr="-R %.2f",
        ),
        no_bias=dict(
            argstr="-N",
        ),
        no_pve=dict(
            argstr="--nopve",
        ),
        number_classes=dict(
            argstr="-n %d",
        ),
        other_priors=dict(
            argstr="-A %s",
        ),
        out_basename=dict(
            argstr="-o %s",
            extensions=None,
        ),
        output_biascorrected=dict(
            argstr="-B",
        ),
        output_biasfield=dict(
            argstr="-b",
        ),
        output_type=dict(),
        probability_maps=dict(
            argstr="-p",
        ),
        segment_iters=dict(
            argstr="-W %d",
        ),
        segments=dict(
            argstr="-g",
        ),
        use_priors=dict(
            argstr="-P",
        ),
        verbose=dict(
            argstr="-v",
        ),
    )
    inputs = FAST.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FAST_outputs():
    output_map = dict(
        bias_field=dict(),
        mixeltype=dict(
            extensions=None,
        ),
        partial_volume_files=dict(),
        partial_volume_map=dict(
            extensions=None,
        ),
        probability_maps=dict(),
        restored_image=dict(),
        tissue_class_files=dict(),
        tissue_class_map=dict(
            extensions=None,
        ),
    )
    outputs = FAST.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import Segment


def test_Segment_inputs():
    input_map = dict(
        affine_regularization=dict(
            field="opts.regtype",
        ),
        bias_fwhm=dict(
            field="opts.biasfwhm",
        ),
        bias_regularization=dict(
            field="opts.biasreg",
        ),
        clean_masks=dict(
            field="output.cleanup",
        ),
        csf_output_type=dict(
            field="output.CSF",
        ),
        data=dict(
            copyfile=False,
            field="data",
            mandatory=True,
        ),
        gaussians_per_class=dict(
            field="opts.ngaus",
        ),
        gm_output_type=dict(
            field="output.GM",
        ),
        mask_image=dict(
            extensions=None,
            field="opts.msk",
        ),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        paths=dict(),
        sampling_distance=dict(
            field="opts.samp",
        ),
        save_bias_corrected=dict(
            field="output.biascor",
        ),
        tissue_prob_maps=dict(
            field="opts.tpm",
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        warp_frequency_cutoff=dict(
            field="opts.warpco",
        ),
        warping_regularization=dict(
            field="opts.warpreg",
        ),
        wm_output_type=dict(
            field="output.WM",
        ),
    )
    inputs = Segment.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Segment_outputs():
    output_map = dict(
        bias_corrected_image=dict(
            extensions=None,
        ),
        inverse_transformation_mat=dict(
            extensions=None,
        ),
        modulated_csf_image=dict(
            extensions=None,
        ),
        modulated_gm_image=dict(
            extensions=None,
        ),
        modulated_input_image=dict(
            deprecated="0.10",
            extensions=None,
            new_name="bias_corrected_image",
        ),
        modulated_wm_image=dict(
            extensions=None,
        ),
        native_csf_image=dict(
            extensions=None,
        ),
        native_gm_image=dict(
            extensions=None,
        ),
        native_wm_image=dict(
            extensions=None,
        ),
        normalized_csf_image=dict(
            extensions=None,
        ),
        normalized_gm_image=dict(
            extensions=None,
        ),
        normalized_wm_image=dict(
            extensions=None,
        ),
        transformation_mat=dict(
            extensions=None,
        ),
    )
    outputs = Segment.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class SphericalCoordinateGenerationInputSpec(CommandLineInputSpec):
    inputAtlasImage = File(
        desc="Input atlas image", exists=True, argstr="--inputAtlasImage %s"
    )
    outputPath = traits.Str(
        desc="Output path for rho, phi and theta images", argstr="--outputPath %s"
    )


class SphericalCoordinateGenerationOutputSpec(TraitedSpec):
    pass


class SphericalCoordinateGeneration(SEMLikeCommandLine):
    """title: Spherical Coordinate Generation

category: Testing.FeatureDetection

description: get the atlas image as input and generates the rho, phi and theta images.

version: 0.1.0.$Revision: 1 $(alpha)

contributor: Ali Ghayoor

"""

    input_spec = SphericalCoordinateGenerationInputSpec
    output_spec = SphericalCoordinateGenerationOutputSpec
    _cmd = " SphericalCoordinateGeneration "
    _outputs_filenames = {}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import VBMSegment


def test_VBMSegment_inputs():
    input_map = dict(
        bias_corrected_affine=dict(
            field="estwrite.output.bias.affine",
            usedefault=True,
        ),
        bias_corrected_native=dict(
            field="estwrite.output.bias.native",
            usedefault=True,
        ),
        bias_corrected_normalized=dict(
            field="estwrite.output.bias.warped",
            usedefault=True,
        ),
        bias_fwhm=dict(
            field="estwrite.opts.biasfwhm",
            usedefault=True,
        ),
        bias_regularization=dict(
            field="estwrite.opts.biasreg",
            usedefault=True,
        ),
        cleanup_partitions=dict(
            field="estwrite.extopts.cleanup",
            usedefault=True,
        ),
        csf_dartel=dict(
            field="estwrite.output.CSF.dartel",
            usedefault=True,
        ),
        csf_modulated_normalized=dict(
            field="estwrite.output.CSF.modulated",
            usedefault=True,
        ),
        csf_native=dict(
            field="estwrite.output.CSF.native",
            usedefault=True,
        ),
        csf_normalized=dict(
            field="estwrite.output.CSF.warped",
            usedefault=True,
        ),
        dartel_template=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
            field="estwrite.extopts.dartelwarp.normhigh.darteltpm",
        ),
        deformation_field=dict(
            field="estwrite.output.warps",
            usedefault=True,
        ),
        display_results=dict(
            field="estwrite.extopts.print",
            usedefault=True,
        ),
        gaussians_per_class=dict(
            usedefault=True,
        ),
        gm_dartel=dict(
            field="estwrite.output.GM.dartel",
            usedefault=True,
        ),
        gm_modulated_normalized=dict(
            field="estwrite.output.GM.modulated",
            usedefault=True,
        ),
        gm_native=dict(
            field="estwrite.output.GM.native",
            usedefault=True,
        ),
        gm_normalized=dict(
            field="estwrite.output.GM.warped",
            usedefault=True,
        ),
        in_files=dict(
            copyfile=False,
            field="estwrite.data",
            mandatory=True,
        ),
        jacobian_determinant=dict(
            field="estwrite.jacobian.warped",
            usedefault=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        mrf_weighting=dict(
            field="estwrite.extopts.mrf",
            usedefault=True,
        ),
        paths=dict(),
        pve_label_dartel=dict(
            field="estwrite.output.label.dartel",
            usedefault=True,
        ),
        pve_label_native=dict(
            field="estwrite.output.label.native",
            usedefault=True,
        ),
        pve_label_normalized=dict(
            field="estwrite.output.label.warped",
            usedefault=True,
        ),
        sampling_distance=dict(
            field="estwrite.opts.samp",
            usedefault=True,
        ),
        spatial_normalization=dict(
            usedefault=True,
        ),
        tissues=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
            field="estwrite.tpm",
        ),
        use_mcr=dict(),
        use_sanlm_denoising_filter=dict(
            field="estwrite.extopts.sanlm",
            usedefault=True,
        ),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        warping_regularization=dict(
            field="estwrite.opts.warpreg",
            usedefault=True,
        ),
        wm_dartel=dict(
            field="estwrite.output.WM.dartel",
            usedefault=True,
        ),
        wm_modulated_normalized=dict(
            field="estwrite.output.WM.modulated",
            usedefault=True,
        ),
        wm_native=dict(
            field="estwrite.output.WM.native",
            usedefault=True,
        ),
        wm_normalized=dict(
            field="estwrite.output.WM.warped",
            usedefault=True,
        ),
    )
    inputs = VBMSegment.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_VBMSegment_outputs():
    output_map = dict(
        bias_corrected_images=dict(),
        dartel_input_images=dict(),
        forward_deformation_field=dict(),
        inverse_deformation_field=dict(),
        jacobian_determinant_images=dict(),
        modulated_class_images=dict(),
        native_class_images=dict(),
        normalized_bias_corrected_images=dict(),
        normalized_class_images=dict(),
        pve_label_native_images=dict(),
        pve_label_normalized_images=dict(),
        pve_label_registered_images=dict(),
        transformation_mat=dict(),
    )
    outputs = VBMSegment.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



"""sphinx autodoc ext."""
from sphinx.locale import _
from sphinx.ext import autodoc
from nipype.interfaces.base import BaseInterface
from .gh import get_url

_ClassDocumenter = autodoc.ClassDocumenter
RST_CLASS_BLOCK = """
.. index:: {name}

.. _{module}.{name}:

{name}
{underline}
`Link to code <{code_url}>`__

"""


class NipypeClassDocumenter(_ClassDocumenter):  # type: ignore
    priority = 20

    def add_directive_header(self, sig: str) -> None:
        if self.doc_as_attr:
            self.directivetype = "attribute"

        # Copied from super
        domain = getattr(self, "domain", "py")
        directive = getattr(self, "directivetype", self.objtype)
        name = self.format_name()
        sourcename = self.get_sourcename()

        is_interface = False
        try:
            is_interface = issubclass(self.object, BaseInterface)
        except TypeError:
            pass

        if is_interface is True:
            lines = RST_CLASS_BLOCK.format(
                code_url=get_url(self.object),
                module=self.modname,
                name=name,
                underline="=" * len(name),
            )
            for line in lines.splitlines():
                self.add_line(line, sourcename)
        else:
            self.add_line(
                ".. %s:%s:: %s%s" % (domain, directive, name, sig), sourcename
            )
            if self.options.noindex:
                self.add_line("   :noindex:", sourcename)
            if self.objpath:
                # Be explicit about the module, this is necessary since .. class::
                # etc. don't support a prepended module name
                self.add_line("   :module: %s" % self.modname, sourcename)

        # add inheritance info, if wanted
        if not self.doc_as_attr and self.options.show_inheritance:
            sourcename = self.get_sourcename()
            self.add_line("", sourcename)
            bases = getattr(self.object, "__bases__", [])
            bases_links = []

            for b in bases:
                based_interface = False
                try:
                    based_interface = issubclass(b, BaseInterface)
                except TypeError:
                    pass

                if b.__module__ in ("__builtin__", "builtins"):
                    bases_links.append(":class:`%s`" % b.__name__)
                elif based_interface:
                    bases_links.append(":ref:`%s.%s`" % (b.__module__, b.__name__))
                else:
                    bases_links.append(":class:`%s.%s`" % (b.__module__, b.__name__))

            self.add_line("   " + _("Bases: %s") % ", ".join(bases_links), sourcename)


def setup(app):
    app.add_autodocumenter(NipypeClassDocumenter)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path
from ...niftyreg.tests.test_regutils import no_nifty_tool
from .. import LabelFusion, CalcTopNCC


@pytest.mark.skipif(
    no_nifty_tool(cmd="seg_LabFusion"), reason="niftyseg is not installed"
)
def test_seg_lab_fusion():
    """ Test interfaces for seg_labfusion"""
    # Create a node object
    steps = LabelFusion()

    # Check if the command is properly defined
    cmd = get_custom_path("seg_LabFusion", env_dir="NIFTYSEGDIR")
    assert steps.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        steps.run()

    # Assign some input data
    in_file = example_data("im1.nii")
    file_to_seg = example_data("im2.nii")
    template_file = example_data("im3.nii")
    steps.inputs.in_file = in_file
    steps.inputs.kernel_size = 2.0
    steps.inputs.file_to_seg = file_to_seg
    steps.inputs.template_file = template_file
    steps.inputs.template_num = 2
    steps.inputs.classifier_type = "STEPS"

    cmd_tmp = "{cmd} -in {in_file} -STEPS 2.000000 2 {file_to_seg} \
{template_file} -out {out_file}"

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        file_to_seg=file_to_seg,
        template_file=template_file,
        out_file="im1_steps.nii",
    )

    assert steps.cmdline == expected_cmd

    # Staple
    staple = LabelFusion(kernel_size=2.0, template_num=2, classifier_type="STAPLE")
    in_file = example_data("im1.nii")
    file_to_seg = example_data("im2.nii")
    template_file = example_data("im3.nii")
    staple.inputs.in_file = in_file
    staple.inputs.file_to_seg = file_to_seg
    staple.inputs.template_file = template_file

    cmd_tmp = "{cmd} -in {in_file} -STAPLE -ALL -out {out_file}"
    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        file_to_seg=file_to_seg,
        template_file=template_file,
        out_file="im1_staple.nii",
    )

    assert staple.cmdline == expected_cmd

    # Assign some input data
    mv_node = LabelFusion(
        template_num=2, classifier_type="MV", sm_ranking="ROINCC", dilation_roi=2
    )
    in_file = example_data("im1.nii")
    file_to_seg = example_data("im2.nii")
    template_file = example_data("im3.nii")
    mv_node.inputs.in_file = in_file
    mv_node.inputs.file_to_seg = file_to_seg
    mv_node.inputs.template_file = template_file

    cmd_tmp = "{cmd} -in {in_file} -MV -ROINCC 2 2 {file_to_seg} \
{template_file} -out {out_file}"

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        file_to_seg=file_to_seg,
        template_file=template_file,
        out_file="im1_mv.nii",
    )

    assert mv_node.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd="seg_CalcTopNCC"), reason="niftyseg is not installed"
)
def test_seg_calctopncc():
    """ Test interfaces for seg_CalctoNCC"""
    # Create a node object
    calctopncc = CalcTopNCC()

    # Check if the command is properly defined
    cmd = get_custom_path("seg_CalcTopNCC", env_dir="NIFTYSEGDIR")
    assert calctopncc.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        calctopncc.run()

    # Assign some input data
    in_file = example_data("im1.nii")
    file1 = example_data("im2.nii")
    file2 = example_data("im3.nii")
    calctopncc.inputs.in_file = in_file
    calctopncc.inputs.num_templates = 2
    calctopncc.inputs.in_templates = [file1, file2]
    calctopncc.inputs.top_templates = 1

    cmd_tmp = "{cmd} -target {in_file} -templates 2 {file1} {file2} -n 1"
    expected_cmd = cmd_tmp.format(cmd=cmd, in_file=in_file, file1=file1, file2=file2,)

    assert calctopncc.cmdline == expected_cmd



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import FieldMap


def test_FieldMap_inputs():
    input_map = dict(
        anat_file=dict(
            copyfile=False,
            extensions=None,
            field="subj.anat",
        ),
        blip_direction=dict(
            field="subj.defaults.defaultsval.blipdir",
            mandatory=True,
        ),
        echo_times=dict(
            field="subj.defaults.defaultsval.et",
            mandatory=True,
        ),
        epi_file=dict(
            copyfile=False,
            extensions=None,
            field="subj.session.epi",
            mandatory=True,
        ),
        epifm=dict(
            field="subj.defaults.defaultsval.epifm",
            usedefault=True,
        ),
        jacobian_modulation=dict(
            field="subj.defaults.defaultsval.ajm",
            usedefault=True,
        ),
        jobtype=dict(
            usedefault=True,
        ),
        magnitude_file=dict(
            copyfile=False,
            extensions=None,
            field="subj.data.presubphasemag.magnitude",
            mandatory=True,
        ),
        mask_fwhm=dict(
            field="subj.defaults.defaultsval.mflags.fwhm",
            usedefault=True,
        ),
        maskbrain=dict(
            field="subj.defaults.defaultsval.maskbrain",
            usedefault=True,
        ),
        matchanat=dict(
            field="subj.matchanat",
            usedefault=True,
        ),
        matchvdm=dict(
            field="subj.matchvdm",
            usedefault=True,
        ),
        matlab_cmd=dict(),
        method=dict(
            field="subj.defaults.defaultsval.uflags.method",
            usedefault=True,
        ),
        mfile=dict(
            usedefault=True,
        ),
        ndilate=dict(
            field="subj.defaults.defaultsval.mflags.ndilate",
            usedefault=True,
        ),
        nerode=dict(
            field="subj.defaults.defaultsval.mflags.nerode",
            usedefault=True,
        ),
        pad=dict(
            field="subj.defaults.defaultsval.uflags.pad",
            usedefault=True,
        ),
        paths=dict(),
        phase_file=dict(
            copyfile=False,
            extensions=None,
            field="subj.data.presubphasemag.phase",
            mandatory=True,
        ),
        reg=dict(
            field="subj.defaults.defaultsval.mflags.reg",
            usedefault=True,
        ),
        sessname=dict(
            field="subj.sessname",
            usedefault=True,
        ),
        template=dict(
            copyfile=False,
            extensions=None,
            field="subj.defaults.defaultsval.mflags.template",
        ),
        thresh=dict(
            field="subj.defaults.defaultsval.mflags.thresh",
            usedefault=True,
        ),
        total_readout_time=dict(
            field="subj.defaults.defaultsval.tert",
            mandatory=True,
        ),
        unwarp_fwhm=dict(
            field="subj.defaults.defaultsval.uflags.fwhm",
            usedefault=True,
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        writeunwarped=dict(
            field="subj.writeunwarped",
            usedefault=True,
        ),
        ws=dict(
            field="subj.defaults.defaultsval.uflags.ws",
            usedefault=True,
        ),
    )
    inputs = FieldMap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FieldMap_outputs():
    output_map = dict(
        vdm=dict(
            extensions=None,
        ),
    )
    outputs = FieldMap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""

Base I/O specifications for Nipype interfaces
.............................................

Define the API for the I/O of interfaces

"""
import os
from inspect import isclass
from copy import deepcopy
from warnings import warn
from packaging.version import Version

from traits.trait_errors import TraitError
from traits.trait_handlers import TraitDictObject, TraitListObject
from ...utils.filemanip import md5, hash_infile, hash_timestamp
from .traits_extension import (
    traits,
    File,
    Str,
    Undefined,
    isdefined,
    has_metadata,
    OutputMultiObject,
)

from ... import config, __version__

_float_fmt = "{:.10f}".format
nipype_version = Version(__version__)


class BaseTraitedSpec(traits.HasTraits):
    """
    Provide a few methods necessary to support nipype interface api

    The inputs attribute of interfaces call certain methods that are not
    available in traits.HasTraits. These are provided here.

    new metadata:

    * usedefault : set this to True if the default value of the trait should be
      used. Unless this is set, the attributes are set to traits.Undefined

    new attribute:

    * get_hashval : returns a tuple containing the state of the trait as a dict
      and hashvalue corresponding to dict.

    XXX Reconsider this in the long run, but it seems like the best
    solution to move forward on the refactoring.
    """

    package_version = nipype_version

    def __init__(self, **kwargs):
        """ Initialize handlers and inputs"""
        # NOTE: In python 2.6, object.__init__ no longer accepts input
        # arguments.  HasTraits does not define an __init__ and
        # therefore these args were being ignored.
        # super(TraitedSpec, self).__init__(*args, **kwargs)
        super(BaseTraitedSpec, self).__init__(**kwargs)
        traits.push_exception_handler(reraise_exceptions=True)
        undefined_traits = {}
        for trait in self.copyable_trait_names():
            if not self.traits()[trait].usedefault:
                undefined_traits[trait] = Undefined
        self.trait_set(trait_change_notify=False, **undefined_traits)
        self._generate_handlers()
        self.trait_set(**kwargs)

    def items(self):
        """ Name, trait generator for user modifiable traits
        """
        for name in sorted(self.copyable_trait_names()):
            yield name, self.traits()[name]

    def __repr__(self):
        """ Return a well-formatted representation of the traits """
        outstr = []
        for name, value in sorted(self.trait_get().items()):
            outstr.append("%s = %s" % (name, value))
        return "\n{}\n".format("\n".join(outstr))

    def _generate_handlers(self):
        """Find all traits with the 'xor' metadata and attach an event
        handler to them.
        """
        has_xor = dict(xor=lambda t: t is not None)
        xors = self.trait_names(**has_xor)
        for elem in xors:
            self.on_trait_change(self._xor_warn, elem)
        has_deprecation = dict(deprecated=lambda t: t is not None)
        deprecated = self.trait_names(**has_deprecation)
        for elem in deprecated:
            self.on_trait_change(self._deprecated_warn, elem)

    def _xor_warn(self, obj, name, old, new):
        """ Generates warnings for xor traits
        """
        if isdefined(new):
            trait_spec = self.traits()[name]
            # for each xor, set to default_value
            for trait_name in trait_spec.xor:
                if trait_name == name:
                    # skip ourself
                    continue
                if isdefined(getattr(self, trait_name)):
                    self.trait_set(
                        trait_change_notify=False, **{"%s" % name: Undefined}
                    )
                    msg = (
                        'Input "%s" is mutually exclusive with input "%s", '
                        "which is already set"
                    ) % (name, trait_name)
                    raise IOError(msg)

    def _deprecated_warn(self, obj, name, old, new):
        """Checks if a user assigns a value to a deprecated trait
        """
        if isdefined(new):
            trait_spec = self.traits()[name]
            msg1 = "Input %s in interface %s is deprecated." % (
                name,
                self.__class__.__name__.split("InputSpec")[0],
            )
            msg2 = (
                "Will be removed or raise an error as of release %s"
                % trait_spec.deprecated
            )
            if trait_spec.new_name:
                if trait_spec.new_name not in self.copyable_trait_names():
                    raise TraitError(
                        msg1 + " Replacement trait %s not found" % trait_spec.new_name
                    )
                msg3 = "It has been replaced by %s." % trait_spec.new_name
            else:
                msg3 = ""
            msg = " ".join((msg1, msg2, msg3))
            if Version(str(trait_spec.deprecated)) < self.package_version:
                raise TraitError(msg)
            else:
                if trait_spec.new_name:
                    msg += "Unsetting old value %s; setting new value %s." % (
                        name,
                        trait_spec.new_name,
                    )
                warn(msg)
                if trait_spec.new_name:
                    self.trait_set(
                        trait_change_notify=False,
                        **{"%s" % name: Undefined, "%s" % trait_spec.new_name: new}
                    )

    def trait_get(self, **kwargs):
        """ Returns traited class as a dict

        Augments the trait get function to return a dictionary without
        notification handles
        """
        out = super(BaseTraitedSpec, self).trait_get(**kwargs)
        out = self._clean_container(out, Undefined)
        return out

    get = trait_get

    def get_traitsfree(self, **kwargs):
        """ Returns traited class as a dict

        Augments the trait get function to return a dictionary without
        any traits. The dictionary does not contain any attributes that
        were Undefined
        """
        out = super(BaseTraitedSpec, self).trait_get(**kwargs)
        out = self._clean_container(out, skipundefined=True)
        return out

    def _clean_container(self, objekt, undefinedval=None, skipundefined=False):
        """Convert a traited obejct into a pure python representation.
        """
        if isinstance(objekt, TraitDictObject) or isinstance(objekt, dict):
            out = {}
            for key, val in list(objekt.items()):
                if isdefined(val):
                    out[key] = self._clean_container(val, undefinedval)
                else:
                    if not skipundefined:
                        out[key] = undefinedval
        elif (
            isinstance(objekt, TraitListObject)
            or isinstance(objekt, list)
            or isinstance(objekt, tuple)
        ):
            out = []
            for val in objekt:
                if isdefined(val):
                    out.append(self._clean_container(val, undefinedval))
                else:
                    if not skipundefined:
                        out.append(undefinedval)
                    else:
                        out.append(None)
            if isinstance(objekt, tuple):
                out = tuple(out)
        else:
            out = None
            if isdefined(objekt):
                out = objekt
            else:
                if not skipundefined:
                    out = undefinedval
        return out

    def has_metadata(self, name, metadata, value=None, recursive=True):
        """
        Return has_metadata for the requested trait name in this
        interface
        """
        return has_metadata(self.trait(name).trait_type, metadata, value, recursive)

    def get_hashval(self, hash_method=None):
        """Return a dictionary of our items with hashes for each file.

        Searches through dictionary items and if an item is a file, it
        calculates the md5 hash of the file contents and stores the
        file name and hash value as the new key value.

        However, the overall bunch hash is calculated only on the hash
        value of a file. The path and name of the file are not used in
        the overall hash calculation.

        Returns
        -------
        list_withhash : dict
            Copy of our dictionary with the new file hashes included
            with each file.
        hashvalue : str
            The md5 hash value of the traited spec

        """
        list_withhash = []
        list_nofilename = []
        for name, val in sorted(self.trait_get().items()):
            if not isdefined(val) or self.has_metadata(name, "nohash", True):
                # skip undefined traits and traits with nohash=True
                continue

            hash_files = not self.has_metadata(
                name, "hash_files", False
            ) and not self.has_metadata(name, "name_source")
            list_nofilename.append(
                (
                    name,
                    self._get_sorteddict(
                        val, hash_method=hash_method, hash_files=hash_files
                    ),
                )
            )
            list_withhash.append(
                (
                    name,
                    self._get_sorteddict(
                        val, True, hash_method=hash_method, hash_files=hash_files
                    ),
                )
            )
        return list_withhash, md5(str(list_nofilename).encode()).hexdigest()

    def _get_sorteddict(
        self, objekt, dictwithhash=False, hash_method=None, hash_files=True
    ):
        if isinstance(objekt, dict):
            out = []
            for key, val in sorted(objekt.items()):
                if isdefined(val):
                    out.append(
                        (
                            key,
                            self._get_sorteddict(
                                val,
                                dictwithhash,
                                hash_method=hash_method,
                                hash_files=hash_files,
                            ),
                        )
                    )
        elif isinstance(objekt, (list, tuple)):
            out = []
            for val in objekt:
                if isdefined(val):
                    out.append(
                        self._get_sorteddict(
                            val,
                            dictwithhash,
                            hash_method=hash_method,
                            hash_files=hash_files,
                        )
                    )
            if isinstance(objekt, tuple):
                out = tuple(out)
        else:
            out = None
            if isdefined(objekt):
                if (
                    hash_files
                    and isinstance(objekt, (str, bytes))
                    and os.path.isfile(objekt)
                ):
                    if hash_method is None:
                        hash_method = config.get("execution", "hash_method")

                    if hash_method.lower() == "timestamp":
                        hash = hash_timestamp(objekt)
                    elif hash_method.lower() == "content":
                        hash = hash_infile(objekt)
                    else:
                        raise Exception("Unknown hash method: %s" % hash_method)
                    if dictwithhash:
                        out = (objekt, hash)
                    else:
                        out = hash
                elif isinstance(objekt, float):
                    out = _float_fmt(objekt)
                else:
                    out = objekt
        return out

    @property
    def __all__(self):
        return self.copyable_trait_names()

    def __getstate__(self):
        """
        Override __getstate__ so that OutputMultiObjects are correctly pickled.

        >>> class OutputSpec(TraitedSpec):
        ...     out = OutputMultiObject(traits.List(traits.Int))
        >>> spec = OutputSpec()
        >>> spec.out = [[4]]
        >>> spec.out
        [4]

        >>> spec.__getstate__()['out']
        [[4]]

        >>> spec.__setstate__(spec.__getstate__())
        >>> spec.out
        [4]

        """
        state = super(BaseTraitedSpec, self).__getstate__()
        for key in self.__all__:
            _trait_spec = self.trait(key)
            if _trait_spec.is_trait_type(OutputMultiObject):
                state[key] = _trait_spec.handler.get_value(self, key)
        return state


class TraitedSpec(BaseTraitedSpec):
    """ Create a subclass with strict traits.

    This is used in 90% of the cases.
    """

    _ = traits.Disallow


class BaseInterfaceInputSpec(TraitedSpec):
    pass


class DynamicTraitedSpec(BaseTraitedSpec):
    """ A subclass to handle dynamic traits

    This class is a workaround for add_traits and clone_traits not
    functioning well together.
    """

    def __deepcopy__(self, memo):
        """
        Replace the ``__deepcopy__`` member with a traits-friendly implementation.

        A bug in ``__deepcopy__`` for ``HasTraits`` results in weird cloning behaviors.
        """
        id_self = id(self)
        if id_self in memo:
            return memo[id_self]
        dup_dict = deepcopy(self.trait_get(), memo)
        # access all keys
        for key in self.copyable_trait_names():
            if key in self.__dict__.keys():
                _ = getattr(self, key)
        # clone once
        dup = self.clone_traits(memo=memo)
        for key in self.copyable_trait_names():
            try:
                _ = getattr(dup, key)
            except:
                pass
        # clone twice
        dup = self.clone_traits(memo=memo)
        dup.trait_set(**dup_dict)
        return dup


class CommandLineInputSpec(BaseInterfaceInputSpec):
    args = Str(argstr="%s", desc="Additional parameters to the command")
    environ = traits.DictStrStr(
        desc="Environment variables", usedefault=True, nohash=True
    )


class StdOutCommandLineInputSpec(CommandLineInputSpec):
    out_file = File(argstr="> %s", position=-1, genfile=True)


class MpiCommandLineInputSpec(CommandLineInputSpec):
    use_mpi = traits.Bool(
        False, desc="Whether or not to run the command with mpiexec", usedefault=True
    )
    n_procs = traits.Int(
        desc="Num processors to specify to mpiexec. Do not "
        "specify if this is managed externally (e.g. through "
        "SGE)"
    )


def get_filecopy_info(cls):
    """Provides information about file inputs to copy or link to cwd.
    Necessary for pipeline operation
    """
    if cls.input_spec is None:
        return None

    # normalize_filenames is not a classmethod, hence check first
    if not isclass(cls) and hasattr(cls, "normalize_filenames"):
        cls.normalize_filenames()
    info = []
    inputs = cls.input_spec() if isclass(cls) else cls.inputs
    metadata = dict(copyfile=lambda t: t is not None)
    for name, spec in sorted(inputs.traits(**metadata).items()):
        info.append(dict(key=name, copy=spec.copyfile))
    return info



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import nipype.interfaces.fsl.dti as fsl
from nipype.interfaces.fsl import Info, no_fsl
from nipype.interfaces.base import Undefined

import pytest
from nipype.testing.fixtures import create_files_in_directory


# test dtifit
@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_dtifit2(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    dti = fsl.DTIFit()
    # make sure command gets called
    assert dti.cmd == "dtifit"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        dti.run()

    # .inputs based parameters setting
    dti.inputs.dwi = filelist[0]
    dti.inputs.base_name = "foo.dti.nii"
    dti.inputs.mask = filelist[1]
    dti.inputs.bvecs = filelist[0]
    dti.inputs.bvals = filelist[1]
    dti.inputs.min_z = 10
    dti.inputs.max_z = 50

    assert (
        dti.cmdline
        == "dtifit -k %s -o foo.dti.nii -m %s -r %s -b %s -Z 50 -z 10"
        % (filelist[0], filelist[1], filelist[0], filelist[1])
    )


@pytest.mark.xfail(reason="These tests are skipped until we clean up some of this code")
def test_randomise2():

    rand = fsl.Randomise()

    # make sure command gets called
    assert rand.cmd == "randomise"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        rand.run()

    # .inputs based parameters setting
    rand.inputs.input_4D = "infile.nii"
    rand.inputs.output_rootname = "outfile"
    rand.inputs.design_matrix = "design.mat"
    rand.inputs.t_contrast = "infile.con"

    actualCmdline = sorted(rand.cmdline.split())
    cmd = "randomise -i infile.nii -o outfile -d design.mat -t infile.con"
    desiredCmdline = sorted(cmd.split())
    assert actualCmdline == desiredCmdline

    # .run based parameter setting
    rand2 = fsl.Randomise(
        input_4D="infile2",
        output_rootname="outfile2",
        f_contrast="infile.f",
        one_sample_gmean=True,
        int_seed=4,
    )

    actualCmdline = sorted(rand2.cmdline.split())
    cmd = "randomise -i infile2 -o outfile2 -1 -f infile.f --seed=4"
    desiredCmdline = sorted(cmd.split())
    assert actualCmdline == desiredCmdline

    rand3 = fsl.Randomise()
    results = rand3.run(input_4D="infile3", output_rootname="outfile3")
    assert results.runtime.cmdline == "randomise -i infile3 -o outfile3"

    # test arguments for opt_map
    opt_map = {
        "demean_data": ("-D", True),
        "one_sample_gmean": ("-1", True),
        "mask_image": ("-m inp_mask", "inp_mask"),
        "design_matrix": ("-d design.mat", "design.mat"),
        "t_contrast": ("-t input.con", "input.con"),
        "f_contrast": ("-f input.fts", "input.fts"),
        "xchange_block_labels": ("-e design.grp", "design.grp"),
        "print_unique_perm": ("-q", True),
        "print_info_parallelMode": ("-Q", True),
        "num_permutations": ("-n 10", 10),
        "vox_pvalus": ("-x", True),
        "fstats_only": ("--fonly", True),
        "thresh_free_cluster": ("-T", True),
        "thresh_free_cluster_2Dopt": ("--T2", True),
        "cluster_thresholding": ("-c 0.20", 0.20),
        "cluster_mass_thresholding": ("-C 0.40", 0.40),
        "fcluster_thresholding": ("-F 0.10", 0.10),
        "fcluster_mass_thresholding": ("-S 0.30", 0.30),
        "variance_smoothing": ("-v 0.20", 0.20),
        "diagnostics_off": ("--quiet", True),
        "output_raw": ("-R", True),
        "output_perm_vect": ("-P", True),
        "int_seed": ("--seed=20", 20),
        "TFCE_height_param": ("--tfce_H=0.11", 0.11),
        "TFCE_extent_param": ("--tfce_E=0.50", 0.50),
        "TFCE_connectivity": ("--tfce_C=0.30", 0.30),
        "list_num_voxel_EVs_pos": ("--vxl=1,2,3,4", "1,2,3,4"),
        "list_img_voxel_EVs": ("--vxf=6,7,8,9,3", "6,7,8,9,3"),
    }

    for name, settings in list(opt_map.items()):
        rand4 = fsl.Randomise(
            input_4D="infile", output_rootname="root", **{name: settings[1]}
        )
        assert rand4.cmdline == rand4.cmd + " -i infile -o root " + settings[0]


@pytest.mark.xfail(reason="These tests are skipped until we clean up some of this code")
def test_Randomise_parallel():
    rand = fsl.Randomise_parallel()

    # make sure command gets called
    assert rand.cmd == "randomise_parallel"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        rand.run()

    # .inputs based parameters setting
    rand.inputs.input_4D = "infile.nii"
    rand.inputs.output_rootname = "outfile"
    rand.inputs.design_matrix = "design.mat"
    rand.inputs.t_contrast = "infile.con"

    actualCmdline = sorted(rand.cmdline.split())
    cmd = "randomise_parallel -i infile.nii -o outfile -d design.mat -t " "infile.con"
    desiredCmdline = sorted(cmd.split())
    assert actualCmdline == desiredCmdline

    # .run based parameter setting
    rand2 = fsl.Randomise_parallel(
        input_4D="infile2",
        output_rootname="outfile2",
        f_contrast="infile.f",
        one_sample_gmean=True,
        int_seed=4,
    )

    actualCmdline = sorted(rand2.cmdline.split())
    cmd = "randomise_parallel -i infile2 -o outfile2 -1 -f infile.f --seed=4"
    desiredCmdline = sorted(cmd.split())
    assert actualCmdline == desiredCmdline

    rand3 = fsl.Randomise_parallel()
    results = rand3.run(input_4D="infile3", output_rootname="outfile3")
    assert results.runtime.cmdline == "randomise_parallel -i infile3 -o outfile3"

    # test arguments for opt_map
    opt_map = {
        "demean_data": ("-D", True),
        "one_sample_gmean": ("-1", True),
        "mask_image": ("-m inp_mask", "inp_mask"),
        "design_matrix": ("-d design.mat", "design.mat"),
        "t_contrast": ("-t input.con", "input.con"),
        "f_contrast": ("-f input.fts", "input.fts"),
        "xchange_block_labels": ("-e design.grp", "design.grp"),
        "print_unique_perm": ("-q", True),
        "print_info_parallelMode": ("-Q", True),
        "num_permutations": ("-n 10", 10),
        "vox_pvalus": ("-x", True),
        "fstats_only": ("--fonly", True),
        "thresh_free_cluster": ("-T", True),
        "thresh_free_cluster_2Dopt": ("--T2", True),
        "cluster_thresholding": ("-c 0.20", 0.20),
        "cluster_mass_thresholding": ("-C 0.40", 0.40),
        "fcluster_thresholding": ("-F 0.10", 0.10),
        "fcluster_mass_thresholding": ("-S 0.30", 0.30),
        "variance_smoothing": ("-v 0.20", 0.20),
        "diagnostics_off": ("--quiet", True),
        "output_raw": ("-R", True),
        "output_perm_vect": ("-P", True),
        "int_seed": ("--seed=20", 20),
        "TFCE_height_param": ("--tfce_H=0.11", 0.11),
        "TFCE_extent_param": ("--tfce_E=0.50", 0.50),
        "TFCE_connectivity": ("--tfce_C=0.30", 0.30),
        "list_num_voxel_EVs_pos": ("--vxl=" + repr([1, 2, 3, 4]), repr([1, 2, 3, 4])),
        "list_img_voxel_EVs": ("--vxf=" + repr([6, 7, 8, 9, 3]), repr([6, 7, 8, 9, 3])),
    }

    for name, settings in list(opt_map.items()):
        rand4 = fsl.Randomise_parallel(
            input_4D="infile", output_rootname="root", **{name: settings[1]}
        )
        assert rand4.cmdline == rand4.cmd + " -i infile -o root " + settings[0]


# test proj_thresh
@pytest.mark.xfail(reason="These tests are skipped until we clean up some of this code")
def test_Proj_thresh():
    proj = fsl.ProjThresh()

    # make sure command gets called
    assert proj.cmd == "proj_thresh"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        proj.run()

    # .inputs based parameters setting
    proj.inputs.volumes = ["vol1", "vol2", "vol3"]
    proj.inputs.threshold = 3
    assert proj.cmdline == "proj_thresh vol1 vol2 vol3 3"

    proj2 = fsl.ProjThresh(threshold=10, volumes=["vola", "volb"])
    assert proj2.cmdline == "proj_thresh vola volb 10"

    # .run based parameters setting
    proj3 = fsl.ProjThresh()
    results = proj3.run(volumes=["inp1", "inp3", "inp2"], threshold=2)
    assert results.runtime.cmdline == "proj_thresh inp1 inp3 inp2 2"
    assert results.runtime.returncode != 0
    assert isinstance(results.interface.inputs.volumes, list)
    assert results.interface.inputs.threshold == 2

    # test arguments for opt_map
    # Proj_thresh doesn't have an opt_map{}


# test vec_reg
@pytest.mark.xfail(reason="These tests are skipped until we clean up some of this code")
def test_Vec_reg():

    vrg = fsl.VecReg()

    # make sure command gets called
    assert vrg.cmd == "vecreg"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        vrg.run()

    # .inputs based parameters setting
    vrg.inputs.infile = "infile"
    vrg.inputs.outfile = "outfile"
    vrg.inputs.refVolName = "MNI152"
    vrg.inputs.affineTmat = "tmat.mat"
    assert vrg.cmdline == "vecreg -i infile -o outfile -r MNI152 -t tmat.mat"

    # .run based parameter setting
    vrg2 = fsl.VecReg(
        infile="infile2",
        outfile="outfile2",
        refVolName="MNI152",
        affineTmat="tmat2.mat",
        brainMask="nodif_brain_mask",
    )

    actualCmdline = sorted(vrg2.cmdline.split())
    cmd = "vecreg -i infile2 -o outfile2 -r MNI152 -t tmat2.mat -m nodif_brain_mask"
    desiredCmdline = sorted(cmd.split())
    assert actualCmdline == desiredCmdline

    vrg3 = fsl.VecReg()
    results = vrg3.run(
        infile="infile3",
        outfile="outfile3",
        refVolName="MNI152",
        affineTmat="tmat3.mat",
    )

    assert (
        results.runtime.cmdline
        == "vecreg -i infile3 -o outfile3 -r MNI152 -t tmat3.mat"
    )
    assert results.runtime.returncode != 0
    assert results.interface.inputs.infile == "infile3"
    assert results.interface.inputs.outfile == "outfile3"
    assert results.interface.inputs.refVolName == "MNI152"
    assert results.interface.inputs.affineTmat == "tmat3.mat"

    # test arguments for opt_map
    opt_map = {
        "verbose": ("-v", True),
        "helpDoc": ("-h", True),
        "tensor": ("--tensor", True),
        "affineTmat": ("-t Tmat", "Tmat"),
        "warpFile": ("-w wrpFile", "wrpFile"),
        "interpolation": ("--interp=sinc", "sinc"),
        "brainMask": ("-m mask", "mask"),
    }

    for name, settings in list(opt_map.items()):
        vrg4 = fsl.VecReg(
            infile="infile",
            outfile="outfile",
            refVolName="MNI152",
            **{name: settings[1]}
        )
        assert (
            vrg4.cmdline == vrg4.cmd + " -i infile -o outfile -r MNI152 " + settings[0]
        )


# test find_the_biggest
@pytest.mark.xfail(reason="These tests are skipped until we clean up some of this code")
def test_Find_the_biggest():
    fbg = fsl.FindTheBiggest()

    # make sure command gets called
    assert fbg.cmd == "find_the_biggest"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        fbg.run()

    # .inputs based parameters setting
    fbg.inputs.infiles = "seed*"
    fbg.inputs.outfile = "fbgfile"
    assert fbg.cmdline == "find_the_biggest seed* fbgfile"

    fbg2 = fsl.FindTheBiggest(infiles="seed2*", outfile="fbgfile2")
    assert fbg2.cmdline == "find_the_biggest seed2* fbgfile2"

    # .run based parameters setting
    fbg3 = fsl.FindTheBiggest()
    results = fbg3.run(infiles="seed3", outfile="out3")
    assert results.runtime.cmdline == "find_the_biggest seed3 out3"

    # test arguments for opt_map
    # Find_the_biggest doesn't have an opt_map{}


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_tbss_skeleton(create_files_in_directory):
    skeletor = fsl.TractSkeleton()

    files, newdir = create_files_in_directory

    # Test the underlying command
    assert skeletor.cmd == "tbss_skeleton"

    # It shouldn't run yet
    with pytest.raises(ValueError):
        skeletor.run()

    # Test the most basic way to use it
    skeletor.inputs.in_file = files[0]

    # First by implicit argument
    skeletor.inputs.skeleton_file = True
    assert skeletor.cmdline == "tbss_skeleton -i a.nii -o %s" % os.path.join(
        newdir, "a_skeleton.nii"
    )

    # Now with a specific name
    skeletor.inputs.skeleton_file = "old_boney.nii"
    assert skeletor.cmdline == "tbss_skeleton -i a.nii -o old_boney.nii"

    # Now test the more complicated usage
    bones = fsl.TractSkeleton(in_file="a.nii", project_data=True)

    # This should error
    with pytest.raises(ValueError):
        bones.run()

    # But we can set what we need
    bones.inputs.threshold = 0.2
    bones.inputs.distance_map = "b.nii"
    bones.inputs.data_file = "b.nii"  # Even though that's silly

    # Now we get a command line
    assert bones.cmdline == "tbss_skeleton -i a.nii -p 0.200 b.nii %s b.nii %s" % (
        Info.standard_image("LowerCingulum_1mm.nii.gz"),
        os.path.join(newdir, "b_skeletonised.nii"),
    )

    # Can we specify a mask?
    bones.inputs.use_cingulum_mask = Undefined
    bones.inputs.search_mask_file = "a.nii"
    assert (
        bones.cmdline
        == "tbss_skeleton -i a.nii -p 0.200 b.nii a.nii b.nii %s"
        % os.path.join(newdir, "b_skeletonised.nii")
    )


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_distancemap(create_files_in_directory):
    mapper = fsl.DistanceMap()

    files, newdir = create_files_in_directory

    # Test the underlying command
    assert mapper.cmd == "distancemap"

    # It shouldn't run yet
    with pytest.raises(ValueError):
        mapper.run()

    # But if we do this...
    mapper.inputs.in_file = "a.nii"

    # It should
    assert mapper.cmdline == "distancemap --out=%s --in=a.nii" % os.path.join(
        newdir, "a_dstmap.nii"
    )

    # And we should be able to write out a maxima map
    mapper.inputs.local_max_file = True
    assert mapper.cmdline == "distancemap --out=%s --in=a.nii --localmax=%s" % (
        os.path.join(newdir, "a_dstmap.nii"),
        os.path.join(newdir, "a_lclmax.nii"),
    )

    # And call it whatever we want
    mapper.inputs.local_max_file = "max.nii"
    assert (
        mapper.cmdline
        == "distancemap --out=%s --in=a.nii --localmax=max.nii"
        % os.path.join(newdir, "a_dstmap.nii")
    )
